@@ -1,6 +1,16 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2008-2012 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2008-2013 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 
 '''
 This module contains the base class for the zim application and the main
@@ -21,7 +31,7 @@ Overview
 
 The script C{zim.py} is a thin wrapper around the L{main()} function
 defined here. THe main function validates commandline options and if
-all is well it either calls the background daemon to connect to some
+all is well it either calls the background process to connect to some
 running instance of zim, or it instantiates a L{NotebookInterface}
 object, or an object of a subclass like L{GtkInterface} (for the
 graphic user interface) or L{WWWInterface} (for the webinterface).
@@ -33,7 +43,7 @@ The graphical user interface is implemented in the L{zim.gui} module
 and it's sub-modules. The webinterface is implemented in L{zim.www}.
 
 The graphical interface uses a background process to coordinate
-between instances, this is implemented in L{zim.daemon}.
+between instances, this is implemented in L{zim.ipc}.
 
 Regardsless of the interface choosen there is a L{Notebook} object
 which implements a generic API for accessing and storing pages and
@@ -84,10 +94,10 @@ For asynchronous actions see L{zim.async}.
 
 
 # Bunch of meta data, used at least in the about dialog
-__version__ = '0.56'
+__version__ = '0.59'
 __url__='http://www.zim-wiki.org'
 __author__ = 'Jaap Karssenberg <jaap.karssenberg@gmail.com>'
-__copyright__ = 'Copyright 2008 - 2012 Jaap Karssenberg <jaap.karssenberg@gmail.com>'
+__copyright__ = 'Copyright 2008 - 2013 Jaap Karssenberg <jaap.karssenberg@gmail.com>'
 __license__='''\
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -109,12 +119,13 @@ from getopt import gnu_getopt, GetoptError
 
 from zim.fs import File, Dir
 from zim.errors import Error
-from zim.config import data_dir, config_file, log_basedirs, ZIM_DATA_DIR, \
-					   XDG_CONFIG_HOME, ConfigDictFile
+from zim.config import data_dir, config_file, get_config, log_basedirs, \
+	ZIM_DATA_DIR, ConfigDictFile
 
 
 logger = logging.getLogger('zim')
 
+
 if ZIM_DATA_DIR:
 	# We are running from a source dir - use the locale data included there
 	localedir = ZIM_DATA_DIR.dir.subdir('locale').path
@@ -137,16 +148,15 @@ ZIM_EXECUTABLE = 'zim'
 longopts = ('verbose', 'debug')
 commands = (
 	'help', 'version', 'gui', 'server', 'export', 'search',
-	'index', 'manual', 'plugin', 'daemon'
+	'index', 'manual', 'plugin', 'ipc-server',
 )
 commandopts = {
-	'gui': ('list', 'geometry=', 'fullscreen', 'no-daemon'),
-	'server': ('port=', 'template=', 'gui', 'no-daemon'),
+	'gui': ('list', 'geometry=', 'fullscreen', 'standalone'),
+	'server': ('port=', 'template=', 'gui', 'standalone'),
 	'export': ('format=', 'template=', 'output=', 'root-url=', 'index-page='),
 	'search': (),
 	'index': ('output=',),
 	'plugin': (),
-	'daemon': (),
 }
 shortopts = {
 	'v': 'version', 'h': 'help',
@@ -174,6 +184,7 @@ General Options:
   --gui           run the editor (this is the default)
   --server        run the web server
   --export        export to a different format
+  --search        run a search query on a notebook
   --index         build an index for a notebook
   --plugin        call a specific plugin function
   --manual        open the user manual
@@ -187,7 +198,7 @@ GUI Options:
                   opening the default notebook
   --geometry      window size and position as WxH+X+Y
   --fullscreen    start in fullscreen mode
-  --no-daemon     start a single instance, no daemon
+  --standalone     start a single instance, no background process
 
 Server Options:
   --port          port to use (defaults to 8080)
@@ -277,6 +288,11 @@ def main(argv):
 		# We were given an absolute path, e.g. "python ./zim.py"
 		ZIM_EXECUTABLE = zim_exec_file.path
 
+	# Check for special commandline args for ipc, does not return
+	# if handled
+	import zim.ipc
+	zim.ipc.handle_argv()
+
 	# Let getopt parse the option list
 	short = ''.join(shortopts.keys())
 	for s, l in shortopts.items():
@@ -406,12 +422,9 @@ def main(argv):
 				notebook = default
 				logger.info('Opening default notebook')
 
-		if 'no_daemon' in optsdict or os.name == 'nt':
+		if 'standalone' in optsdict:
 			import zim.gui
-			try:
-				del optsdict['no_daemon']
-			except KeyError:
-				pass
+			del optsdict['standalone']
 			if not notebook:
 				import zim.gui.notebookdialog
 				notebook = zim.gui.notebookdialog.prompt_notebook()
@@ -420,43 +433,35 @@ def main(argv):
 			handler = zim.gui.GtkInterface(notebook, page, **optsdict)
 			handler.main()
 		else:
-			import zim.daemon
-			proxy = zim.daemon.DaemonProxy()
+			from zim.ipc import start_server_if_not_running, ServerProxy
 			if not notebook:
-				# Need to call this after spawning the daemon, else we
-				# have gtk loaded in the daemon process, and that causes
-				# problems with using gtk in child processes.
 				import zim.gui.notebookdialog
 				notebook = zim.gui.notebookdialog.prompt_notebook()
 				if not notebook:
-					proxy.quit_if_nochild()
 					return # User canceled notebook dialog
-			gui = proxy.get_notebook(notebook)
 
+			start_server_if_not_running()
+			server = ServerProxy()
+			gui = server.get_notebook(notebook)
 			gui.present(page, **optsdict)
 
 			logger.debug('''
 NOTE FOR BUG REPORTS:
 	At this point zim has send the command to open a notebook to a
-	background process and the current process will no quit.
+	background process and the current process will now quit.
 	If this is the end of your debug output it is probably not useful
 	for bug reports. Please close all zim windows, quit the
 	zim trayicon (if any), and try again.
 ''')
 	elif cmd == 'server':
-		try:
-			del optsdict['no_daemon']
-		except KeyError:
-			pass
+		if 'standalone' in optsdict:
+			del optsdict['standalone']
+			# No daemon support for server, so no option doesn;t
+			# do anything for now
 
 		import zim.www
 		handler = zim.www.Server(*args, **optsdict)
 		handler.main()
-	elif cmd == 'daemon':
-		# just start the daemon, nothing else
-		import zim.daemon
-		proxy = zim.daemon.DaemonProxy()
-		proxy.ping()
 	elif cmd == 'plugin':
 		import zim.plugins
 		try:
@@ -464,7 +469,7 @@ NOTE FOR BUG REPORTS:
 		except IndexError:
 			raise UsageError
 		module = zim.plugins.get_plugin_module(pluginname)
-		module.main(None, *args)
+		module.main(*args)
 
 
 def ZimCmd():
@@ -476,7 +481,7 @@ def ZimCmd():
 	# needed
 	from zim.applications import Application
 	if ZIM_EXECUTABLE.endswith('.exe'):
-		return Application(ZIM_EXECUTABLE)
+		return Application((ZIM_EXECUTABLE,))
 	elif sys.executable:
 		# If not an compiled executable, we assume it is python
 		# (Application class does this automatically for python scripts
@@ -536,7 +541,7 @@ class NotebookInterface(gobject.GObject):
 		self.notebook = None
 		self.plugins = []
 
-		self.preferences = config_file('preferences.conf')
+		self.preferences = get_config('preferences.conf')
 		self.preferences['General'].setdefault('plugins',
 			['calendar', 'insertsymbol', 'printtobrowser', 'versioncontrol'])
 
@@ -556,9 +561,13 @@ class NotebookInterface(gobject.GObject):
 		# but sort them here to make behavior predictable.
 		plugins = self.preferences['General']['plugins']
 		for name in sorted(plugins):
-			klass = zim.plugins.get_plugin(name)
-			if klass.is_profile_independent:
-				self.load_plugin(name)
+			try:
+				klass = zim.plugins.get_plugin(name)
+			except:
+				logger.exception('Failed to load plugin klass for plugin "%s"', name)
+			else:
+				if klass.is_profile_independent:
+					self.load_plugin(name)
 
 	def load_plugins(self, independent_only=False):
 		'''Loads all the plugins defined in the preferences that are
@@ -600,7 +609,7 @@ class NotebookInterface(gobject.GObject):
 				raise AssertionError, 'Dependencies failed for plugin %s' % name
 			plugin = klass(self)
 		except:
-			logger.exception('Failed to load plugin %s', name)
+			logger.exception('Failed to load plugin "%s"', name)
 			try:
 				self.preferences['General']['plugins'].remove(name)
 				self.preferences.set_modified(True)
@@ -609,7 +618,7 @@ class NotebookInterface(gobject.GObject):
 			return None
 		else:
 			self.plugins.append(plugin)
-			logger.debug('Loaded plugin %s (%s)', name, plugin)
+			logger.debug('Loaded plugin "%s" (%s)', name, plugin)
 
 		plugin.plugin_key = name
 		if not name in self.preferences['General']['plugins']:
@@ -666,7 +675,7 @@ class NotebookInterface(gobject.GObject):
 					independent.append(plugin.plugin_key)
 
 			if independent:
-				default = config_file('preferences.conf')
+				default = get_config('preferences.conf')
 				for name in independent:
 					if name not in default['General']['plugins']:
 						default['General']['plugins'].append(name)
@@ -778,14 +787,14 @@ class NotebookInterface(gobject.GObject):
 			# we cary over any settings from the current one
 			logger.debug('Profile changed to: %s', notebook.profile)
 			basename = self.notebook.profile.lower() + '.conf'
-			file = XDG_CONFIG_HOME.file(('zim', 'profiles', basename))
+			file = config_file(('profiles', basename))
 			self.preferences.change_file(file)
 			self.preferences.write()
 		else:
 			# Load default preferences
 			# We do a full flush to reset to default
 			logger.debug('Profile reset to default')
-			preferences = config_file('preferences.conf')
+			preferences = get_config('preferences.conf')
 			file = preferences.file
 			self.preferences.change_file(file)
 			for section in self.preferences.values():
