@@ -18,7 +18,8 @@ import gobject
 import zim.fs
 import zim.errors
 
-from zim.parsing import split_quoted_strings
+from zim.parsing import split_quoted_strings, is_uri_re, is_win32_path_re
+from zim.config import get_environ_list
 
 
 logger = logging.getLogger('zim.applications')
@@ -78,6 +79,14 @@ class Application(object):
 		self.cmd = tuple(cmd)
 		self.tryexeccmd = tryexeccmd
 
+	def __repr__(self):
+		if hasattr(self, 'key'):
+			return '<%s: %s>' % (self.__class__.__name__, self.key)
+		elif hasattr(self, 'cmd'):
+			return '<%s: %s>' % (self.__class__.__name__, self.cmd)
+		else:
+			return '<%s: %s>' % (self.__class__.__name__, self.name)
+
 	@property
 	def name(self):
 		return self.cmd[0]
@@ -96,7 +105,7 @@ class Application(object):
 				return None
 		else:
 			# lookup in PATH
-			for dir in os.environ['PATH'].split(os.pathsep):
+			for dir in get_environ_list('PATH'):
 				file = os.sep.join((dir, cmd))
 				if zim.fs.isfile(file):
 					return file
@@ -122,7 +131,14 @@ class Application(object):
 
 	def _checkargs(self, cwd, args):
 		assert args is None or isinstance(args, (tuple, list))
-		argv = [a.encode(zim.fs.ENCODING) for a in self._cmd(args)]
+		argv = self._cmd(args)
+
+		# if it is a python script, insert interpreter as the executable
+		if argv[0].endswith('.py') and sys.executable:
+			argv = list(argv)
+			argv.insert(0, sys.executable)
+
+		argv = [a.encode(zim.fs.ENCODING) for a in argv]
 		if cwd:
 			cwd = unicode(cwd).encode(zim.fs.ENCODING)
 		return cwd, argv
@@ -137,7 +153,24 @@ class Application(object):
 		'''
 		cwd, argv = self._checkargs(cwd, args)
 		logger.info('Running: %s (cwd: %s)', argv, cwd)
-		p = subprocess.Popen(argv, cwd=cwd, stdout=open(os.devnull, 'w'), stderr=subprocess.PIPE)
+		if os.name == 'nt':
+			# http://code.activestate.com/recipes/409002/
+			info = subprocess.STARTUPINFO()
+			try:
+				info.dwFlags |= subprocess.STARTF_USESHOWWINDOW
+			except AttributeError:
+				info.dwFlags |= 1 # STARTF_USESHOWWINDOW = 0x01
+
+			p = subprocess.Popen(argv,
+				cwd=cwd,
+				stdout=open(os.devnull, 'w'),
+				stderr=subprocess.PIPE,
+				startupinfo=info)
+		else:
+			p = subprocess.Popen(argv,
+				cwd=cwd,
+				stdout=open(os.devnull, 'w'),
+				stderr=subprocess.PIPE)
 		p.wait()
 		stderr = p.stderr.read()
 
@@ -172,8 +205,9 @@ class Application(object):
 		if stderr:
 			logger.warn(stderr)
 
-		return [line + '\n' for line in stdout.splitlines()]
+		return map(unicode, [line + '\n' for line in stdout.splitlines()])
 			# Explicit newline conversion, e.g. on windows \r\n -> \n
+			# FIXME Assume local encoding is respected (!?)
 
 	def spawn(self, args=None, callback=None, data=None, cwd=None):
 		'''Start the application in the background and return immediately.
@@ -195,9 +229,6 @@ class Application(object):
 		@returns: the PID for the new process
 		'''
 		cwd, argv = self._checkargs(cwd, args)
-		# if it is a python script, insert interpreter as the executable
-		if argv[0].endswith('.py'):
-			argv.insert(0, sys.executable)
 		opts = {}
 
 		flags = gobject.SPAWN_SEARCH_PATH
@@ -210,9 +241,8 @@ class Application(object):
 			pid, stdin, stdout, stderr = \
 				gobject.spawn_async(argv, flags=flags, **opts)
 		except gobject.GError:
-			logger.exception('Failed running: %s', argv)
-			#~ name = self.name
-			#~ ErrorDialog(None, _('Could not run application: %s') % name).run()
+			from zim.gui.widgets import ErrorDialog
+			ErrorDialog(None, _('Failed running: %s') % argv[0]).run()
 				#~ # T: error when application failed to start
 			return None
 		else:
@@ -264,7 +294,7 @@ class WebBrowser(Application):
 
 class StartFile(Application):
 	'''Application wrapper for C{os.startfile()}. Can be used on
-	windows to open files with the default application.
+	windows to open files and URLs with the default application.
 	'''
 
 	name = _('Default') + ' (os)' # T: label for default application
@@ -286,7 +316,15 @@ class StartFile(Application):
 		if callback:
 			logger.warn('os.startfile does not support a callback')
 
-		for file in args:
-			path = os.path.normpath(unicode(file))
+		for arg in args:
+			if isinstance(arg, (zim.fs.File, zim.fs.Dir)):
+				path = os.path.normpath(arg.path)
+			elif is_uri_re.match(arg) and not is_win32_path_re.match(arg):
+				# URL or e.g. mailto: or outlook: URI
+				path = unicode(arg)
+			else:
+				# must be file
+				path = os.path.normpath(unicode(arg))
+
 			logger.info('Opening with os.startfile: %s', path)
 			os.startfile(path)
