@@ -31,8 +31,8 @@ Supported tags:
 	- mark for highlighted text, renderd with background color or underlined
 	- strike for text that is removed, usually renderd as strike through
 	- code for inline verbatim text
-	- ul for bullet lists
-	- .. for checkbox lists
+	- ul for bullet and checkbox lists
+	- ol for numbered lists
 	- li for list items
 	- link for links, attribute href gives the target
 	- img for images, attributes src, width, height an optionally href
@@ -48,9 +48,7 @@ markup it is not allowed to nest elements in arbitrary ways.
 
 TODO: allow links to be nested in other elements
 TODO: allow strike to have sub elements
-TODO: allow classes to set hints for visual rendering and other interaction
 TODO: add HR element
-TODO: ol for numbered lists
 
 If a page starts with a h1 this heading is considered the page title,
 else we can fall back to the page name as title.
@@ -62,6 +60,7 @@ to a title or subtitle in the document.
 '''
 
 import re
+import string
 import logging
 
 from zim.fs import Dir, File
@@ -85,18 +84,17 @@ except:
 
 try:
 	import xml.etree.cElementTree as ElementTreeModule
-	from xml.etree.cElementTree import \
-		Element, SubElement, TreeBuilder
+	from xml.etree.cElementTree import Element, SubElement
 except:  # pragma: no cover
 	logger.warn('Could not load cElementTree, defaulting to ElementTree')
 	import xml.etree.ElementTree as ElementTreeModule
-	from xml.etree.ElementTree import \
-		Element, SubElement, TreeBuilder
+	from xml.etree.ElementTree import Element, SubElement
 
 
 EXPORT_FORMAT = 1
 IMPORT_FORMAT = 2
 NATIVE_FORMAT = 4
+TEXT_FORMAT = 8 # Used for "Copy As" menu - these all prove "text/plain" mimetype
 
 UNCHECKED_BOX = 'unchecked-box'
 CHECKED_BOX = 'checked-box'
@@ -104,13 +102,48 @@ XCHECKED_BOX = 'xchecked-box'
 BULLET = '*'
 
 
+def increase_list_iter(listiter):
+	'''Get the next item in a list for a numbered list
+	E.g if C{listiter} is C{"1"} this function returns C{"2"}, if it
+	is C{"a"} it returns C{"b"}.
+	@param listiter: the current item, either an integer number or
+	single letter
+	@returns: the next item, or C{None}
+	'''
+	try:
+		i = int(listiter)
+		return str(i + 1)
+	except ValueError:
+		try:
+			i = string.letters.index(listiter)
+			return string.letters[i+1]
+		except ValueError: # listiter is not a letter
+			return None
+		except IndexError: # wrap to start of list
+			return string.letters[0]
+
+
+
 def list_formats(type):
 	if type == EXPORT_FORMAT:
-		return ['HTML','LaTeX']
+		return ['HTML','LaTeX', 'Markdown (pandoc)', 'RST (sphinx)']
+	elif type == TEXT_FORMAT:
+		return ['Text', 'Wiki', 'Markdown (pandoc)', 'RST (sphinx)']
 	else:
 		assert False, 'TODO'
 
 
+def canonical_name(name):
+	# "HTML" -> html
+	# "Markdown (pandoc)" -> "markdown"
+	# "Text" -> "plain"
+	name = name.lower()
+	if ' ' in name:
+		name, _ = name.split(' ', 1)
+	if name == 'text': return 'plain'
+	else: return name
+
+
 def get_format(name):
 	'''Returns the module object for a specific format.'''
 	# If this method is removes, class names in formats/*.py can be made more explicit
@@ -124,15 +157,15 @@ def get_format_module(name):
 	@param name: the format name
 	@returns: a module object
 	'''
-	return zim.plugins.get_module('zim.formats', name)
+	return zim.plugins.get_module('zim.formats', canonical_name(name))
 
 
 def get_parser(name, *arg, **kwarg):
 	'''Returns a parser object instance for a specific format
 
 	@param name: format name
-
-	All other param are passed on to the object constructor
+	@param arg: arguments to pass to the parser object
+	@param kwarg: keyword arguments to pass to the parser object
 
 	@returns: parser object instance (subclass of L{ParserClass})
 	'''
@@ -145,8 +178,8 @@ def get_dumper(name, *arg, **kwarg):
 	'''Returns a dumper object instance for a specific format
 
 	@param name: format name
-
-	All other param are passed on to the object constructor
+	@param arg: arguments to pass to the dumper object
+	@param kwarg: keyword arguments to pass to the dumper object
 
 	@returns: dumper object instance (subclass of L{DumperClass})
 	'''
@@ -155,6 +188,27 @@ def get_dumper(name, *arg, **kwarg):
 	return klass(*arg, **kwarg)
 
 
+from xml.etree.ElementTree import TreeBuilder as _TreeBuilder
+
+class TreeBuilder(_TreeBuilder):
+	# Hack to deal with API incompatibility between versions of etree
+	# Note that cElementTree.TreeBuilder is a function, so we can not
+	# subclass it :(  therefore using the python version with the
+	# current element class as factory (which might be the c variant)
+
+	def __init__(self):
+		_TreeBuilder.__init__(self, Element)
+
+	def start(self, tag, attrs=None):
+		if attrs is None:
+			attrs = {}
+		_TreeBuilder.start(self, tag, attrs)
+
+	def data(self, data):
+		assert isinstance(data, basestring), 'Got: %s' % data
+		_TreeBuilder.data(self, data)
+
+
 class ParseTree(ElementTreeModule.ElementTree):
 	'''Wrapper for zim parse trees, derives from ElementTree.'''
 
@@ -227,19 +281,32 @@ class ParseTree(ElementTreeModule.ElementTree):
 		'''Parsing from file is not implemented, use fromstring() instead'''
 		raise NotImplementedError
 
+	def _get_heading_element(self, level=1):
+		children = self.getroot().getchildren()
+		if children:
+			first = children[0]
+			if first.tag == 'h' and first.attrib['level'] >= level:
+				return first
+		return None
+
+	def get_heading(self, level=1):
+		heading_elem = self._get_heading_element(level)
+		if heading_elem is not None:
+			return heading_elem.text
+		else:
+			return ""
+
 	def set_heading(self, text, level=1):
 		'''Set the first heading of the parse tree to 'text'. If the tree
 		already has a heading of the specified level or higher it will be
 		replaced. Otherwise the new heading will be prepended.
 		'''
 		root = self.getroot()
-		children = root.getchildren()
 		tail = "\n"
-		if children:
-			first = children[0]
-			if first.tag == 'h' and first.attrib['level'] >= level:
-				tail = first.tail # Keep trailing text
-				root.remove(first)
+		heading_elem = self._get_heading_element(level)
+		if heading_elem is not None:
+			tail = heading_elem.tail
+			root.remove(heading_elem)
 		heading = Element('h', {'level': level})
 		heading.text = text
 		heading.tail = tail
@@ -279,6 +346,14 @@ class ParseTree(ElementTreeModule.ElementTree):
 				filepath = element.attrib['src']
 				element.attrib['_src_file'] = notebook.resolve_file(element.attrib['src'], path)
 
+	def unresolve_images(self):
+		'''Undo effect of L{resolve_images()}, mainly intended for
+		testing.
+		'''
+		for element in self.getiterator('img'):
+			if '_src_file' in element.attrib:
+				element.attrib.pop('_src_file')
+
 	def encode_urls(self, mode=URL_ENCODE_READABLE):
 		'''Calls encode_url() on all links that contain urls.
 		See zim.parsing for details. Modifies the parse tree.
@@ -447,6 +522,7 @@ class ParseTreeBuilder(object):
 					self._last.tail = None
 			else:
 				self._last = self._stack[-1]
+				self._tail = False
 				if not self._last.text is None:
 					self._data = [self._last.text]
 					self._last.text = None
@@ -673,6 +749,18 @@ class BaseLinker(object):
 		'''Set whether the format supports relative files links or not'''
 		self.usebase = usebase
 
+	def resolve_file(self, link):
+		'''Find the source file for an attachment
+		Used e.g. by the latex format to find files for equations to
+		be inlined. Do not use this method to resolve links, the file
+		given here might be temporary and is not guaranteed to be
+		available after the export. Use L{link()} or C{link_file()}
+		to resolve links to files.
+		@returns: a L{File} object or C{None} if no file was found
+		@implementation: must be implemented by child classes
+		'''
+		raise NotImplementedError
+
 	def link(self, link):
 		'''Returns an url for a link in a zim page
 		This method is used to translate links of any type. It determined
@@ -689,7 +777,7 @@ class BaseLinker(object):
 		@returns: url, uri or whatever link notation is relevant in the
 		context of this linker
 		@rtype: string
-		 '''
+		'''
 		assert not self.path is None
 		if not link in self._links:
 			type = link_type(link)
@@ -707,6 +795,8 @@ class BaseLinker(object):
 				else:
 					logger.warn('No URL found for interwiki link "%s"', link)
 					link = href
+			elif type == 'notebook':
+				href = self.link_notebook(link)
 			else: # I dunno, some url ?
 				method = 'link_' + type
 				if hasattr(self, method):
@@ -731,11 +821,15 @@ class BaseLinker(object):
 		raise NotImplementedError
 
 	def link_page(self, link):
-		'''To be overloaded, return an url for a page link'''
+		'''To be overloaded, return an url for a page link
+		@implementation: must be implemented by child classes
+		'''
 		raise NotImplementedError
 
 	def link_file(self, path):
-		'''To be overloaded, return an url for a file link'''
+		'''To be overloaded, return an url for a file link
+		@implementation: must be implemented by child classes
+		'''
 		raise NotImplementedError
 
 	def link_mailto(self, uri):
@@ -745,3 +839,30 @@ class BaseLinker(object):
 	def link_notebook(self, url):
 		'''Optional method, default just returns url'''
 		return url
+
+
+class StubLinker(BaseLinker):
+	'''Linker used for testing - just gives back the link as it was
+	parsed. DO NOT USE outside of testing.
+	'''
+
+	def __init__(self):
+		BaseLinker.__init__(self)
+		self.path = '<PATH>'
+		self.base = Dir('<NOBASE>')
+
+	def resolve_file(self, link):
+		return self.base.file(link)
+			# Very simple stub, allows finding files be rel path for testing
+
+	def icon(self, name):
+		return 'icon:' + name
+
+	def resource(self, path):
+		return path
+
+	def link_page(self, link):
+		return link
+
+	def link_file(self, path):
+		return path
