@@ -334,6 +334,17 @@ class ScrolledHBox(gtk.HBox):
 		for child in children[last+1:]:
 			child.set_child_visible(False)
 
+	def do_focus(self, direction):
+		# Overrule navigation for <Ctrl><Tab> while leaving
+		# navigation with <Left> and <Right> in tact
+		# (so do not "sub-navigate" with <Ctrl><Tab>).
+		# Otherwise the user has to tab through all buttons before
+		# he can tab to the next widget.
+		if direction in (gtk.DIR_TAB_FORWARD, gtk.DIR_TAB_BACKWARD) \
+		and self.focus_child is not None:
+			return False # Let outer container go to next widget
+		else:
+			return gtk.HBox.do_focus(self, direction)
 
 # Need to register classes defining gobject signals
 gobject.type_register(ScrolledHBox)
@@ -454,7 +465,8 @@ class PathBar(ScrolledHBox):
 			return True
 
 	def on_button_popup_menu(self, button):
-		menu = self.ui.uimanager.get_widget('/page_popup')
+		menu = gtk.Menu()
+		self.ui.populate_popup('page_popup', menu, button.zim_path)
 		menu.popup(None, None, None, 3, 0)
 		return True
 
@@ -487,7 +499,7 @@ class RecentPathBar(PathBar):
 
 	def get_paths(self):
 		# TODO enforce max number of paths shown
-		paths = list(self.history.get_unique())
+		paths = list(self.history.get_recent())
 		paths.reverse()
 		return paths
 
