@@ -8,27 +8,46 @@ import re
 
 
 def split_quoted_strings(string, unescape=True):
-	'''Split a word list respecting quotes.'''
+	'''Split a word list respecting quotes
+
+	This function always expect full words to be quoted, even if quotes
+	appear in the middle of a word, they are considered word
+	boundries.
+
+	( XDG Desktop Entry spec says full words must be quoted and
+	quotes in a word escaped, but doesn't specifify what to do with
+	loose quotes in a string. )
+
+	Also a comma "," is handled specially and is always considered a
+	word on it's own.
+
+	@param string: string to split in words
+	@param unescape: if C{True} quotes are removed, else they are
+	left in place
+	@returns: list of strings
+	'''
 	word_re = Re(r'''
 		(	'(\\'|[^'])*' |  # single quoted word
 			"(\\"|[^"])*" |  # double quoted word
-			[^\s,]+       |  # word without spaces and commas
-			,                # comma
+			[^\s,'"]+     |  # word without spaces and commas
+			,                # comma - (allow "words,word"<<)
 		)''', re.X)
 	string = string.strip()
 	words = []
 	while word_re.match(string):
-		words.append(word_re[1])
+		words.append(word_re[0])
 		i = word_re.m.end()
 		string = string[i:].lstrip()
 	assert not string
 	if unescape:
-		words = map(unescape_quoted_string, words)
+		words = [unescape_quoted_string(w) for w in words]
 	return words
 
 
 def unescape_quoted_string(string):
-	'''Removes quotes from a string and unescapes embedded quotes.'''
+	'''Removes quotes from a string and unescapes embedded quotes
+	@returns: string
+	'''
 	escape_re = re.compile(r'(\\(\\)|\\([\'\"]))')
 	def replace(m):
 		return m.group(2) or m.group(3)
@@ -110,7 +129,7 @@ def url_encode(url, mode=URL_ENCODE_PATH):
 		- C{URL_ENCODE_READABLE}: encode whitespace and all unicode characters
 
 	The mode URL_ENCODE_READABLE can be applied to urls that are already
-	encoded because they do not touch the "%" character. The modes
+	encoded because it does not touch the "%" character. The modes
 	URL_ENCODE_DATA and URL_ENCODE_PATH can only be applied to strings
 	that are known not to be encoded.
 
@@ -312,8 +331,10 @@ class Re(object):
 		return self.m.end(group)
 
 # Some often used regexes
+is_uri_re = Re('^(\w[\w\+\-\.]*):')
+	# "scheme:"
 is_url_re = Re('^(\w[\w\+\-\.]*)://')
-	# scheme "://"
+	# "scheme://"
 is_email_re = Re('^(mailto:\S+|[^\s:]+)\@\S+\.\w+$', re.U)
 	# "mailto:" address
 	# name "@" host
@@ -345,8 +366,21 @@ url_re = Re(r'''(
 	# See rfc/3986 for the official -but unpractical- regex
 
 
+def uri_scheme(link):
+	'''Function that returns a scheme for URIs, URLs and email addresses'''
+	if is_email_re.match(link):
+		return 'mailto'
+	elif is_uri_re.match(link):
+		# Includes URLs, but also URIs like "mid:", "cid:"
+		return is_uri_re[1]
+	else:
+		return None
+
+
 def link_type(link):
 	'''Function that returns a link type for urls and page links'''
+	# More strict than uri_scheme() because page links conflict with
+	# URIs without "//" or without "@"
 	if is_url_re.match(link):
 		if link.startswith('zim+'): type = 'notebook'
 		else: type = is_url_re[1]
@@ -380,6 +414,6 @@ class TextBuffer(list):
 
 	def prefix_lines(self, prefix):
 		'''Prefix each line with string 'prefix'.'''
-		lines = self.get_lines()
+		lines = self.get_lines(end_with_newline=False)
+			# allowing end_with_newline here modifies content
 		self[:] = [prefix + line for line in lines]
-
