@@ -7,7 +7,7 @@
 from zim.plugins import PluginClass
 
 from zim.fs import Dir
-from zim.config import config_file
+from zim.config import get_config
 from zim.applications import Application
 
 
@@ -34,7 +34,7 @@ This is a core plugin shipping with zim.
 		@param uri: a notebook uri
 		@returns: a config dict
 		'''
-		config = config_file('automount.conf')
+		config = get_config('automount.conf')
 		groups = [k for k in config.keys() if k.startswith('Path')]
 		for group in groups:
 			path = group[4:].strip() # len('Path') = 4
