@@ -1,24 +1,28 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2009 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2009-2013 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 import gobject
 import gtk
 
 import re
-import datetime
+
+import logging
 
 from zim.plugins import PluginClass
-from zim.gui.widgets import ui_environment, Dialog, Button
+import zim.datetimetz as datetime
+from zim.datetimetz import dates_for_week, weekcalendar
+from zim.gui.widgets import ui_environment, Dialog, Button, \
+	WindowSidePaneWidget, LEFT_PANE, TOP, WIDGET_POSITIONS
 from zim.notebook import Path
-from zim.templates import TemplateManager
+from zim.templates import TemplateManager, TemplateFunction
+
+
+logger = logging.getLogger('zim.plugins.calendar')
 
 
 # FUTURE: Use calendar.HTMLCalendar from core libs to render this plugin in www
 
-# TODO implement template for calendar pages
-#  - take into account month and year nodes as well
-
 
 ui_xml = '''
 <ui>
@@ -58,11 +62,12 @@ ui_actions = (
 KEYVALS_ENTER = map(gtk.gdk.keyval_from_name, ('Return', 'KP_Enter', 'ISO_Enter'))
 KEYVALS_SPACE = (gtk.gdk.unicode_to_keyval(ord(' ')),)
 
-date_path_re = re.compile(r'^(.*:)?\d{4}:\d{2}:\d{2}$')
+date_path_re = re.compile(r'^(.*:)?\d{4}:\d{1,2}:\d{2}$')
 week_path_re = re.compile(r'^(.*:)?\d{4}:Week \d{2}$')
-month_path_re = re.compile(r'^(.*:)?\d{4}:\d{2}$')
+month_path_re = re.compile(r'^(.*:)?\d{4}:\d{1,2}$')
 year_path_re = re.compile(r'^(.*:)?\d{4}$')
 
+
 def daterange_from_path(path):
 	'''Determine the calendar dates mapped by a specific page
 	@param path: a L{Path} object
@@ -81,12 +86,7 @@ def daterange_from_path(path):
 		type = 'week'
 		year, week = path.name.rsplit(':', 2)[-2:]
 		year, week = map(int, (year, week[5:])) # Assumes "Week XX" string
-		date = datetime.date(year, 1, 1)
-		if week > 0:
-			date = date + datetime.timedelta(week*7 - datetime.date(year, 1, 1).weekday() - 1)
-			end_date = date + datetime.timedelta(6)
-		else:
-			end_date = date + datetime.timedelta(6 - datetime.date(year, 1, 1).weekday() - 1)
+		date, end_date = dates_for_week(year, week)
 	elif month_path_re.match(path.name):
 		type = 'month'
 		year, month = map(int, path.name.rsplit(':', 2)[-2:])
@@ -101,26 +101,25 @@ def daterange_from_path(path):
 		date = datetime.date(year, 1, 1)
 		end_date = datetime.date(year, 12, 31)
 	else:
-		return None# Not a calendar path
+		return None # Not a calendar path
 	return type, date, end_date
 
 
 class CalendarPlugin(PluginClass):
 
 	plugin_info = {
-		'name': _('Calendar'), # T: plugin name
+		'name': _('Journal'), # T: plugin name
 		'description': _('''\
-This plugin turns one namespace into a calendar
-keeping one page per day. A dialog is added with a
-month view of this special namespace.
-
-This is a core plugin shipping with zim.
-'''), # T: plugin description
+This plugin turns one namespace into a journal
+with a page per day, week or month.
+Also adds a calendar widget to access these pages.
+'''),
+		# T: plugin description
 		'author': 'Jaap Karssenberg',
-		'help': 'Plugins:Calendar',
+		'help': 'Plugins:Journal',
 	}
 
-	global DAY, WEEK, MONTH, YEAR # Hack
+	global DAY, WEEK, MONTH, YEAR # Hack - to make sure translation is loaded
 	DAY = _('Day') # T: option value
 	WEEK = _('Week') # T: option value
 	MONTH = _('Month') # T: option value
@@ -129,9 +128,11 @@ This is a core plugin shipping with zim.
 	plugin_preferences = (
 		# key, type, label, default
 		('embedded', 'bool', _('Show calendar in sidepane instead of as dialog'), False), # T: preferences option
-		('granularity', 'choice', _('Use a page for each'), DAY, [DAY, WEEK, MONTH, YEAR]), # T: preferences option, values will be "Day", "Month", ...
-		('namespace', 'namespace', _('Namespace'), ':Calendar'), # T: input label
+		('pane', 'choice', _('Position in the window'), (LEFT_PANE, TOP), WIDGET_POSITIONS), # T: preferences option
+		('granularity', 'choice', _('Use a page for each'), DAY, (DAY, WEEK, MONTH, YEAR)), # T: preferences option, values will be "Day", "Month", ...
+		('namespace', 'namespace', _('Namespace'), ':Journal'), # T: input label
 	)
+	# TODO disable pane setting if not embedded
 
 	def __init__(self, ui):
 		PluginClass.__init__(self, ui)
@@ -143,7 +144,8 @@ This is a core plugin shipping with zim.
 		if self.ui.ui_type == 'gtk':
 			self.ui.add_actions(ui_actions, self)
 			self.ui.add_ui(ui_xml, self)
-			TemplateManager.connect('process-page', self.on_process_page_template)
+			self.connectto(TemplateManager, 'process-page', self.on_process_page_template)
+			self.connectto(self.ui, 'open-page')
 
 	def finalize_notebook(self, notebook):
 		self.do_preferences_changed()
@@ -160,16 +162,24 @@ This is a core plugin shipping with zim.
 		self.disconnect_embedded_widget()
 		PluginClass.disconnect(self)
 
+	def on_open_page(self, ui, page, path):
+		if self.sidepane_widget:
+			self.sidepane_widget.set_page(path)
+		# else dialog takes care of itself
+
 	def connect_embedded_widget(self):
-		from zim.gui.widgets import LEFT_PANE, TOP
 		if not self.sidepane_widget:
 			self.sidepane_widget = CalendarPluginWidget(self)
-			self.sidepane_widget.show_all()
-			self.ui.mainwindow.add_widget(self.sidepane_widget, LEFT_PANE, TOP)
+		else:
+			self.ui.mainwindow.remove(self.sidepane_widget)
+
+		self.ui.mainwindow.add_widget(self.sidepane_widget, self.preferences['pane'])
+		self.sidepane_widget.show_all()
 
 	def disconnect_embedded_widget(self):
 		if self.sidepane_widget:
 			self.ui.mainwindow.remove(self.sidepane_widget)
+			self.sidepane_widget.destroy()
 			self.sidepane_widget = None
 
 	def do_preferences_changed(self):
@@ -186,13 +196,13 @@ This is a core plugin shipping with zim.
 				ns = self.preferences['namespace'].name
 				self.preferences['namespace'] = ns
 			else:
-				self.preferences.setdefault('namespace', ':Calendar')
+				self.preferences.setdefault('namespace', ':Journal')
 				ns = self.preferences['namespace']
 				ns = self.ui.notebook.resolve_path(ns)
 				ns = ns.name
 				self.preferences['namespace'] = ns
 
-			self.ui.notebook.namespace_properties[ns]['template'] = 'Calendar'
+			self.ui.notebook.namespace_properties[ns]['template'] = 'Journal'
 			self._set_template = ns
 
 		if self.ui.ui_type == 'gtk':
@@ -214,7 +224,8 @@ This is a core plugin shipping with zim.
 		if self.preferences['granularity'] == DAY:
 			path = date.strftime('%Y:%m:%d')
 		elif self.preferences['granularity'] == WEEK:
-			path = date.strftime('%Y:Week %U') # FIXME - should this be translated ??
+			year, week, day = weekcalendar(date)
+			path = '%i:Week %02i' % (year, week)
 		elif self.preferences['granularity'] == MONTH:
 			path = date.strftime('%Y:%m')
 		elif self.preferences['granularity'] == YEAR:
@@ -227,10 +238,12 @@ This is a core plugin shipping with zim.
 						+ ':' + date.strftime('%Y:%m') )
 
 	def date_from_path(self, path):
-		'''Returns a datetime.date object for a calendar page'''
+		'''Returns the date for a specific path or C{None}'''
 		dates = daterange_from_path(path)
-		assert dates, 'Not a date path: %s' % path.name
-		return dates[1]
+		if dates:
+			return dates[1]
+		else:
+			return None
 
 	def on_process_page_template(self, manager, template, page, dict):
 		'''Callback called when parsing a template, e.g. when exposing a page
@@ -239,10 +252,13 @@ This is a core plugin shipping with zim.
 		'''
 		daterange = daterange_from_path(page)
 		if daterange:
+			type, start, end = daterange
 			dict['calendar_plugin'] = {
-				'page_type': daterange[0],
-				'date': daterange[1],
-				'end_date': daterange[2],
+				'page_type': type,
+				'date': start,
+				'start_date': start,
+				'end_date': end,
+				'days': DateRangeTemplateFunction(start, end),
 			}
 
 	def suggest_link(self, source, text):
@@ -271,6 +287,22 @@ This is a core plugin shipping with zim.
 	# TODO: Overload the "Insert date" dialog by adding a 'link' option
 
 
+class DateRangeTemplateFunction(TemplateFunction):
+	'''Function to be used in templates to iterate a range of dates'''
+
+	def __init__(self, start, end):
+		self.start = start
+		self.end = end
+
+	def __call__(self, dict):
+		oneday = datetime.timedelta(days=1)
+		yield self.start
+		next = self.start + oneday
+		while next <= self.end:
+			yield next
+			next += oneday
+
+
 class Calendar(gtk.Calendar):
 	'''Custom calendar widget class. Adds an 'activate' signal for when a
 	date is selected explicitly by the user.
@@ -315,16 +347,26 @@ class Calendar(gtk.Calendar):
 gobject.type_register(Calendar)
 
 
-class CalendarPluginWidget(gtk.VBox):
+class CalendarPluginWidget(gtk.VBox, WindowSidePaneWidget):
 
 	def __init__(self, plugin):
 		gtk.VBox.__init__(self)
 		self.plugin = plugin
 
-		format = _('%A %d %B %Y').replace(' 0', ' ') # T: strftime format for current date label
-		label = gtk.Label(datetime.date.today().strftime(str(format)))
-			# str() needed for python 2.5 compatibility
-		self.pack_start(label, False)
+		self.label_box = gtk.HBox()
+		self.pack_start(self.label_box, False)
+
+		self.label = gtk.Label()
+		self.label_box.add(self.label)
+		self._refresh_label()
+		self._timer_id = \
+			gobject.timeout_add(300000, self._refresh_label)
+			# 5 minute = 300_000 ms
+			# Ideally we only need 1 timer per day at 00:00, but not
+			# callback for that
+		self.connect('destroy',
+			lambda o: gobject.source_remove(o._timer_id) )
+			# Clear reference, else we get a new timer for every dialog
 
 		self.calendar = Calendar()
 		self.calendar.display_options(
@@ -336,9 +378,26 @@ class CalendarPluginWidget(gtk.VBox):
 		self.on_month_changed(self.calendar)
 		self.pack_start(self.calendar, False)
 
-		self.plugin.ui.connect('open-page', self.on_open_page)
 		self._select_date_cb = None
 
+	def embed_closebutton(self, button):
+		if button:
+			self.label_box.pack_end(button, False)
+		else:
+			for widget in self.label_box.get_children():
+				if not widget == self.label:
+					self.label_box.remove(widget)
+		return True
+
+	def _refresh_label(self, *a):
+		#print "UPDATE LABEL %s" % id(self)
+		format = _('%A %d %B %Y').replace(' 0', ' ')
+			# T: strftime format for current date label
+		text = datetime.date.today().strftime(str(format))
+			# str() needed for python 2.5 compatibility strftime
+		self.label.set_text(text)
+		return True # else timer is stopped
+
 	def set_select_date_callback(self, func):
 		self._select_date_cb = func
 
@@ -354,15 +413,15 @@ class CalendarPluginWidget(gtk.VBox):
 		namespace = self.plugin.path_for_month_from_date( calendar.get_date() )
 		for path in self.plugin.ui.notebook.index.list_pages(namespace):
 			if date_path_re.match(path.name):
-				date = self.plugin.date_from_path(path)
-				calendar.mark_day(date.day)
+				dates = daterange_from_path(path)
+				if dates and dates[0] == 'day':
+					calendar.mark_day(dates[1].day)
 
-	def on_open_page(self, ui, page, path):
-		try:
-			date = self.plugin.date_from_path(path)
-			self.calendar.select_month(date.month-1, date.year)
-		except AssertionError:
-			pass
+	def set_page(self, page):
+		dates = daterange_from_path(page)
+		if dates and dates[0] != 'year':
+			# Calendar is per month, so do not switch view for year page
+			self.calendar.select_month(dates[1].month-1, dates[1].year)
 
 	def select_date(self, date):
 		self.calendar.select_date(date)
@@ -386,6 +445,11 @@ class CalendarDialog(Dialog):
 		self.action_area.reorder_child(button, 0)
 		self.dateshown = datetime.date.today()
 
+		self.connectto(self.plugin.ui, 'open-page')
+
+	def on_open_page(self, ui, page, path):
+		self.calendar_widget.set_page(page)
+
 	def on_select_date(self, date):
 		if ui_environment['platform'] == 'maemo':
 			# match the user usage pattern
