@@ -8,11 +8,12 @@ import re
 from datetime import date as dateclass
 
 from zim.plugins import PluginClass
-from zim.config import config_file, data_file
+from zim.config import get_config, data_file
 from zim.notebook import resolve_notebook, get_notebook, Notebook, PageNameError
-from zim.daemon import DaemonProxy
-from zim.gui.widgets import Dialog, scrolled_text_view, IconButton, \
+from zim.ipc import start_server_if_not_running, ServerProxy
+from zim.gui.widgets import Dialog, ScrolledTextView, IconButton, \
 	InputForm, gtk_window_set_default_icon
+from zim.gui.clipboard import Clipboard, SelectionClipboard
 from zim.gui.notebookdialog import NotebookComboBox
 from zim.templates import GenericTemplate, StrftimeFunction
 
@@ -26,25 +27,24 @@ usagehelp = '''\
 usage: zim --plugin quicknote [OPTIONS]
 
 Options:
-  notebook=URI       Select the notebook in the dialog
-  namespace=STRING   Fill in the namespace in the dialog
-  basename=STRING    Fill in the page name in the dialog
-  text=TEXT          Provide the text directly
-  input=stdin        Provide the text on stdin
-  input=clipboard    Take the text from the clipboard
-  encoding=base64    Text is encoded in base64
-  encoding=url       Text is url encoded
-                     (In both cases expects utf-8 after decoding)
-  option:url=STRING  Set template parameter
+  notebook=URI         Select the notebook in the dialog
+  page=STRING          Fill in full page name
+  namespace=STRING     Fill in the namespace in the dialog
+  basename=STRING      Fill in the page name in the dialog
+  append=[true|false]  Set whether to append or create new page
+  text=TEXT            Provide the text directly
+  input=stdin          Provide the text on stdin
+  input=clipboard      Take the text from the clipboard
+  encoding=base64      Text is encoded in base64
+  encoding=url         Text is url encoded
+                       (In both cases expects UTF-8 after decoding)
+  attachments=FOLDER   Import all files in FOLDER as attachments,
+                       wiki input can refer these files relatively
+  option:url=STRING    Set template parameter
 '''
 
 
-def main(daemonproxy, *args):
-	assert daemonproxy is None, 'Not intended as daemon child'
-
-	import os
-	assert not os.name == 'nt', 'RPC not supported on windows'
-
+def main(*args):
 	options = {}
 	template_options = {}
 	for arg in args:
@@ -70,14 +70,20 @@ def main(daemonproxy, *args):
 	else:
 		notebook = None
 
+	if 'append' in options:
+		if options['append'].lower() == 'true':
+			options['append'] = True
+		else:
+			options['append'] = False
+
 	if 'input' in options:
 		if options['input'] == 'stdin':
 			import sys
 			text = sys.stdin.read()
 		elif options['input'] == 'clipboard':
 			text = \
-				gtk.Clipboard(selection='PRIMARY').wait_for_text() \
-				or gtk.Clipboard(selection='CLIPBOARD').wait_for_text()
+				SelectionClipboard.get_text() \
+				or Clipboard.get_text()
 	else:
 		text = options.get('text')
 
@@ -100,7 +106,11 @@ def main(daemonproxy, *args):
 	dialog = QuickNoteDialog(None,
 		notebook,
 		options.get('namespace'), options.get('basename'),
-		text, template_options )
+		options.get('append'),
+		text,
+		template_options,
+		options.get('attachments')
+	)
 	dialog.run()
 
 
@@ -153,24 +163,30 @@ This is a core plugin shipping with zim.
 class BoundQuickNoteDialog(Dialog):
 	'''Dialog bound to a specific notebook'''
 
-	def __init__(self, ui, namespace=None, basename=None, text=None, template_options=None):
+	def __init__(self, ui, page=None, namespace=None, basename=None, append=None, text=None, template_options=None, attachments=None):
 		Dialog.__init__(self, ui, _('Quick Note'))
 		self._updating_title = False
 		self._title_set_manually = not basename is None
+		self.attachments = attachments
 
 		self.uistate.setdefault('namespace', None, basestring)
 		namespace = namespace or self.uistate['namespace']
 
 		self.form = InputForm(notebook=self.ui.notebook)
 		self.vbox.pack_start(self.form, False)
-		self._init_inputs(namespace, basename, text, template_options)
+		self._init_inputs(namespace, basename, append, text, template_options)
 
-	def _init_inputs(self, namespace, basename, text, template_options, custom=None):
+	def _init_inputs(self, namespace, basename, append, text, template_options, custom=None):
 		if template_options is None:
 			template_options = {}
 		else:
 			template_options = template_options.copy()
 
+		if namespace is not None and basename is not None:
+			page = namespace + ':' + basename
+		else:
+			page = namespace or basename
+
 		self.form.add_inputs( (
 				('page', 'page', _('Page')),
 				('namespace', 'namespace', _('Namespace')), # T: text entry field
@@ -178,9 +194,9 @@ class BoundQuickNoteDialog(Dialog):
 				('basename', 'string', _('Title')) # T: text entry field
 			) )
 		self.form.update({
-				'page': namespace,
+				'page': page,
 				'namespace': namespace,
-				'new_page': False,
+				'new_page': True,
 				'basename': basename,
 			} )
 
@@ -194,7 +210,10 @@ class BoundQuickNoteDialog(Dialog):
 		# toggling the checkbox
 		self.form.widgets['page'].set_no_show_all(True)
 		self.form.widgets['namespace'].set_no_show_all(True)
-		self.form['new_page'] = bool(self.uistate['new_page'])
+		if append is None:
+			self.form['new_page'] = bool(self.uistate['new_page'])
+		else:
+			self.form['new_page'] = not append
 
 		def switch_input(*a):
 			if self.form['new_page']:
@@ -217,7 +236,7 @@ class BoundQuickNoteDialog(Dialog):
 
 		# Add the main textview and hook up the basename field to
 		# sync with first line of the textview
-		window, textview = scrolled_text_view()
+		window, textview = ScrolledTextView()
 		self.textview = textview
 		self.textview.set_editable(True)
 		self.vbox.add(window)
@@ -226,7 +245,7 @@ class BoundQuickNoteDialog(Dialog):
 		self.textview.get_buffer().connect('changed', self.on_text_changed)
 
 		# Initialize text from template
-		file = data_file('templates/_quicknote.txt')
+		file = data_file('templates/plugins/quicknote.txt')
 		template = GenericTemplate(file.readlines(), name=file)
 		template_options.update({
 			'text': text or '',
@@ -255,6 +274,7 @@ class BoundQuickNoteDialog(Dialog):
 			self.uistate['namespace'] = self.form['page']
 
 	def on_title_changed(self, o):
+		o.set_input_valid(True)
 		if not self._updating_title:
 			self._title_set_manually = True
 
@@ -276,6 +296,10 @@ class BoundQuickNoteDialog(Dialog):
 			self._updating_title = False
 
 	def do_response_ok(self, get_ui=None):
+		# NOTE: Keep in mind that this method should also work using
+		# a proxy object for the ui. This is why we have the get_ui()
+		# argument to construct a proxy.
+
 		buffer = self.textview.get_buffer()
 		bounds = buffer.get_bounds()
 		text = buffer.get_text(*bounds)
@@ -283,14 +307,21 @@ class BoundQuickNoteDialog(Dialog):
 		if self.form['new_page']:
 			if not self.form.widgets['namespace'].get_input_valid() \
 			or not self.form['basename']:
+				if not self.form['basename']:
+					entry = self.form.widgets['basename']
+					entry.set_input_valid(False, show_empty_invalid=True)
 				return False
 
 			if get_ui: ui = get_ui()
 			else: ui = self.ui
+			if ui is None:
+				return False
+
 			path = self.form['namespace'].name + ':' + self.form['basename']
-			ui.new_page_from_text(text, path)
-			if self.open_page.get_active():
-				ui.present(path) # also works with proxy
+			ui.new_page_from_text(text, path,
+				attachments=self.attachments,
+				open_page=self.open_page.get_active()
+			)
 		else:
 			if not self.form.widgets['page'].get_input_valid() \
 			or not self.form['page']:
@@ -298,7 +329,12 @@ class BoundQuickNoteDialog(Dialog):
 
 			if get_ui: ui = get_ui()
 			else: ui = self.ui
+			if ui is None:
+				return False
+
 			path = self.form['page'].name
+			if self.attachments:
+				ui.import_attachments(path, self.attachments)
 			ui.append_text_to_page(path, '\n----\n'+text)
 			if self.open_page.get_active():
 				ui.present(path) # also works with proxy
@@ -309,13 +345,14 @@ class BoundQuickNoteDialog(Dialog):
 class QuickNoteDialog(BoundQuickNoteDialog):
 	'''Dialog which includes a notebook chooser'''
 
-	def __init__(self, ui, notebook=None, namespace=None, basename=None, text=None, template_options=None):
-		self.config = config_file('quicknote.conf')
+	def __init__(self, ui, notebook=None, namespace=None, basename=None, append=None, text=None, template_options=None, attachments=None):
+		self.config = get_config('quicknote.conf')
 		self.uistate = self.config['QuickNoteDialog']
 
 		Dialog.__init__(self, ui, _('Quick Note'))
 		self._updating_title = False
 		self._title_set_manually = not basename is None
+		self.attachments = attachments
 
 		if notebook and not isinstance(notebook, basestring):
 			notebook = notebook.uri
@@ -338,7 +375,7 @@ class QuickNoteDialog(BoundQuickNoteDialog):
 		self.notebookcombobox.connect('changed', self.on_notebook_changed)
 		self.form.attach(self.notebookcombobox, 1,2, 0,1)
 
-		self._init_inputs(namespace, basename, text, template_options)
+		self._init_inputs(namespace, basename, append, text, template_options)
 
 		self.uistate['lastnotebook'] = notebook
 		self._set_autocomplete(notebook)
@@ -348,10 +385,11 @@ class QuickNoteDialog(BoundQuickNoteDialog):
 		self.uistate['lastnotebook'] = notebook
 		self.uistate['new_page'] = self.form['new_page']
 		self.uistate['open_page'] = self.open_page.get_active()
-		if self.uistate['new_page']:
-			self.config['Namespaces'][notebook] = self.form['namespace']
-		else:
-			self.config['Namespaces'][notebook] = self.form['page']
+		if notebook is not None:
+			if self.uistate['new_page']:
+				self.config['Namespaces'][notebook] = self.form['namespace']
+			else:
+				self.config['Namespaces'][notebook] = self.form['page']
 		self.config.write()
 
 	def on_notebook_changed(self, o):
@@ -381,13 +419,11 @@ class QuickNoteDialog(BoundQuickNoteDialog):
 
 	def do_response_ok(self):
 		def get_ui():
-			# HACK to start daemon from separate process
-			# we are not allowed to fork since we already loaded gtk
-			from subprocess import check_call
-			from zim import ZIM_EXECUTABLE
-			check_call([ZIM_EXECUTABLE, '--daemon'])
-
+			start_server_if_not_running()
 			notebook = self.notebookcombobox.get_notebook()
-			return DaemonProxy().get_notebook(notebook)
+			if notebook:
+				return ServerProxy().get_notebook(notebook)
+			else:
+				return None
 
 		return BoundQuickNoteDialog.do_response_ok(self, get_ui)
