@@ -1,12 +1,16 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2008 Jaap Karssenberg <pardus@cpan.org>
+# Copyright 2008 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''Spell check plugin based on gtkspell'''
 
+import os
 import gobject
 
+from zim.config import get_environ
 from zim.plugins import PluginClass
+from zim.gui.widgets import ErrorDialog
+from zim.signals import SIGNAL_AFTER
 
 try:
 	import gtkspell
@@ -59,11 +63,11 @@ This is a core plugin shipping with zim.
 		if self.ui.ui_type == 'gtk':
 			self.ui.add_toggle_actions(ui_toggle_actions, self)
 			self.ui.add_ui(ui_xml, self)
-			self.ui.connect_after('open-page', self.do_open_page)
+			self.connectto(self.ui, 'open-page', order=SIGNAL_AFTER)
 
 	@classmethod
 	def check_dependencies(klass):
-		return [('gtkspell',not gtkspell is None)]	
+		return (not gtkspell is None), [('gtkspell', not gtkspell is None, True)]
 
 	def toggle_spellcheck(self, enable=None):
 		action = self.actiongroup.get_action('toggle_spellcheck')
@@ -80,25 +84,37 @@ This is a core plugin shipping with zim.
 
 		textview = self.ui.mainwindow.pageview.view
 		if enable:
-			# TODO check language in page / notebook / default
 			if self.spell is None:
 				lang = self.preferences['language'] or None
-				self.spell = gtkspell.Spell(textview, lang)
-				textview.gtkspell = self.spell # used by hardcoded hook in pageview
+				try:
+					self.spell = gtkspell.Spell(textview, lang)
+				except:
+					lang = lang or get_environ('LANG') or get_environ('LANGUAGE')
+					ErrorDialog(self.ui, (
+						_('Could not load spell checking for language: "%s"') % lang,
+							# T: error message - %s is replaced with language codes like "en", "en_US"m or "nl_NL"
+						_('This could mean you don\'t have the proper\ndictionaries installed')
+							# T: error message explanation
+					) ).run()
+					return
+				else:
+					textview.gtkspell = self.spell # HACK used by hardcoded hook in pageview
 			else:
 				pass
 		else:
 			if self.spell is None:
 				pass
 			else:
-				self.spell.detach()
+				if textview.gtkspell \
+				and textview.gtkspell == self.spell:
+					textview.gtkspell.detach()
+					textview.gtkspell = None
 				self.spell = None
-				textview.gtkspell = None
 
 		self.uistate['active'] = enable
 		return False # we can be called from idle event
 
-	def do_open_page(self, ui, page, record):
+	def on_open_page(self, ui, page, record):
 		# Assume the old object is detached by hard coded
 		# hook in TextView, just attach a new one.
 		# Use idle timer to avoid lag in page loading.
