@@ -13,8 +13,9 @@ import datetime
 
 from zim.plugins import PluginClass
 from zim.notebook import Path
-from zim.gui.widgets import LEFT_PANE, TOP, BrowserTreeView, populate_popup_add_separator
+from zim.gui.widgets import LEFT_PANE, PANE_POSITIONS, BrowserTreeView, populate_popup_add_separator
 from zim.gui.pageview import FIND_REGEX, SCROLL_TO_MARK_MARGIN, _is_heading_tag
+from zim.signals import ConnectorMixin
 
 
 # FIXME, these methods should be supported by pageview - need anchors - now it is a HACK
@@ -22,10 +23,11 @@ _is_heading = lambda iter: bool(filter(_is_heading_tag, iter.get_tags()))
 
 def find_heading(buffer, heading):
 	'''Find a heading
+	@param buffer: the C{gtk.TextBuffer}
 	@param heading: text of the heading
 	@returns: a C{gtk.TextIter} for the new cursor position or C{None}
 	'''
-	regex = "^%s$" % heading
+	regex = "^%s$" % re.escape(heading)
 	with buffer.tmp_cursor():
 		if buffer.finder.find(regex, FIND_REGEX):
 			iter = buffer.get_insert_iter()
@@ -74,9 +76,12 @@ This is a core plugin shipping with zim.
 
 	plugin_preferences = (
 		# key, type, label, default
+		('pane', 'choice', _('Position in the window'), LEFT_PANE, PANE_POSITIONS),
+			# T: option for plugin preferences
 		('floating', 'bool', _('Show ToC as floating widget instead of in sidepane'), False),
 			# T: option for plugin preferences
 	)
+	# TODO disable pane setting if not embedded
 
 	def __init__(self, ui):
 		PluginClass.__init__(self, ui)
@@ -105,14 +110,18 @@ This is a core plugin shipping with zim.
 	def connect_sidepane(self):
 		if not self.sidepane_widget:
 			self.sidepane_widget = ToCWidget(self.ui)
-			self.ui.mainwindow.add_tab(
-				_('ToC'), self.sidepane_widget, LEFT_PANE)
-				# T: widget label
-			self.sidepane_widget.show_all() # FIXME - should not be needed
+		else:
+			self.ui.mainwindow.remove(self.sidepane_widget)
+
+		self.ui.mainwindow.add_tab(
+			_('ToC'), self.sidepane_widget, self.preferences['pane'])
+			# T: widget label
+		self.sidepane_widget.show_all()
 
 	def disconnect_sidepane(self):
 		if self.sidepane_widget:
 			self.ui.mainwindow.remove(self.sidepane_widget)
+			self.sidepane_widget.disconnect_all()
 			self.sidepane_widget.destroy()
 			self.sidepane_widget = None
 
@@ -124,6 +133,7 @@ This is a core plugin shipping with zim.
 
 	def disconnect_floating(self):
 		if self.floating_widget:
+			self.floating_widget.widget.disconnect_all()
 			self.floating_widget.destroy()
 			self.floating_widget = None
 
@@ -139,10 +149,12 @@ class FloatingToC(gtk.Frame):
 		self.set_shadow_type(gtk.SHADOW_OUT)
 		self.set_size_request(250, -1) # Fixed width
 
+		self.widget = ToCWidget(ui)
+
 		exp = gtk.Expander(_('ToC'))
 		# TODO add mnemonic
 		self.add(exp)
-		exp.add(ToCWidget(ui))
+		exp.add(self.widget)
 
 	def attach(self, textview):
 		# Need to wrap in event box to make widget visible - not sure why
@@ -160,7 +172,7 @@ class FloatingToC(gtk.Frame):
 		textview.add_child_in_window(event_box, gtk.TEXT_WINDOW_WIDGET, 300, 10)
 
 
-class ToCWidget(gtk.ScrolledWindow):
+class ToCWidget(ConnectorMixin, gtk.ScrolledWindow):
 
 	def __init__(self, ui):
 		gtk.ScrolledWindow.__init__(self)
@@ -177,8 +189,8 @@ class ToCWidget(gtk.ScrolledWindow):
 		self.treeview.connect('row-activated', self.on_heading_activated)
 		self.treeview.connect('populate-popup', self.on_populate_popup)
 
-		ui.connect('open-page', self.on_open_page)
-		ui.notebook.connect('stored-page', self.on_stored_page)
+		self.connectto(ui, 'open-page')
+		self.connectto(ui.notebook, 'stored-page')
 		if ui.page:
 			self.on_open_page(ui, ui.page, Path(ui.page.name))
 
@@ -217,11 +229,10 @@ class ToCWidget(gtk.ScrolledWindow):
 		else:
 			return False
 
-	def select_section(buffer, path):
+	def select_section(self, buffer, path):
 		'''Select all text between two headings
-		@param start: starting heading
-		@param end: heading of next section
-		@returns: C{True} if succesfull
+		@param buffer: the C{gtk.TextBuffer} to select in
+		@param path: the C{gtk.TreePath} for the heading of the section
 		'''
 		model = self.treeview.get_model()
 		starttext = model[path][TEXT_COL].decode('utf-8')
@@ -272,7 +283,7 @@ class ToCWidget(gtk.ScrolledWindow):
 	def on_promote(self, *a):
 		# Promote selected paths and all their children
 		model, paths = self.treeview.get_selection().get_selected_rows()
-		if not self.can_demote(paths):
+		if not self.can_promote(paths):
 			return False
 
 		seen = set()
@@ -308,7 +319,7 @@ class ToCWidget(gtk.ScrolledWindow):
 		# Demote selected paths and all their children
 		# note can not demote below level 6
 		model, paths = self.treeview.get_selection().get_selected_rows()
-		if not self.can_promote(paths):
+		if not self.can_demote(paths):
 			return False
 
 		seen = set()
