@@ -18,10 +18,10 @@ from zim.notebook import Path
 from zim.gui.widgets import ui_environment, \
 	Dialog, MessageDialog, \
 	InputEntry, Button, IconButton, MenuButton, \
-	BrowserTreeView, SingleClickTreeView, \
+	BrowserTreeView, SingleClickTreeView, ScrolledWindow, HPaned, \
 	encode_markup_text, decode_markup_text
 from zim.gui.clipboard import Clipboard
-from zim.async import DelayedCallback
+from zim.signals import DelayedCallback, SIGNAL_AFTER
 from zim.formats import get_format, UNCHECKED_BOX, CHECKED_BOX, XCHECKED_BOX
 from zim.config import check_class_allow_empty
 
@@ -52,8 +52,8 @@ ui_xml = '''
 </ui>
 '''
 
-SQL_FORMAT_VERSION = (0, 5)
-SQL_FORMAT_VERSION_STRING = "0.5"
+SQL_FORMAT_VERSION = (0, 6)
+
 
 SQL_CREATE_TABLES = '''
 create table if not exists tasklist (
@@ -65,17 +65,18 @@ create table if not exists tasklist (
 	actionable BOOLEAN,
 	prio INTEGER,
 	due TEXT,
+	tags TEXT,
 	description TEXT
 );
 '''
 
 
-tag_re = re.compile(r'(?<!\S)@(\w+)\b', re.U)
-date_re = re.compile(r'\s*\[d:(.+)\]')
+_tag_re = re.compile(r'(?<!\S)@(\w+)\b', re.U)
+_date_re = re.compile(r'\s*\[d:(.+)\]')
 
 
 _NO_DATE = '9999' # Constant for empty due date - value chosen for sorting properties
-
+_NO_TAGS = '__no_tags__' # Constant that serves as the "no tags" tag - _must_ be lower case
 
 # FUTURE: add an interface for this plugin in the WWW frontend
 
@@ -113,14 +114,23 @@ This is a core plugin shipping with zim.
 			# T: label for plugin preferences dialog
 		('tag_by_page', 'bool', _('Turn page name into tags for task items'), False),
 			# T: label for plugin preferences dialog
-		('deadline_by_page', 'bool', _('Implicit deadline for task items in calendar pages'), False),
+		('deadline_by_page', 'bool', _('Implicit due date for task items in calendar pages'), False),
+			# T: label for plugin preferences dialog
+		('use_workweek', 'bool', _('Flag tasks due on Monday or Tuesday before the weekend'), True),
 			# T: label for plugin preferences dialog
 		('labels', 'string', _('Labels marking tasks'), 'FIXME, TODO', check_class_allow_empty),
 			# T: label for plugin preferences dialog - labels are e.g. "FIXME", "TODO", "TASKS"
 		('next_label', 'string', _('Label for next task'), 'Next:', check_class_allow_empty),
 			# T: label for plugin preferences dialog - label is by default "Next"
+		('nonactionable_tags', 'string', _('Tags for non-actionable tasks'), '', check_class_allow_empty),
+			# T: label for plugin preferences dialog
+		('included_subtrees', 'string', _('Subtree(s) to index'), '', check_class_allow_empty),
+			# T: subtree to search for tasks - default is the whole tree (empty string means everything)
+		('excluded_subtrees', 'string', _('Subtree(s) to ignore'), '', check_class_allow_empty),
+			# T: subtrees of the included subtrees to *not* search for tasks - default is none
 	)
-	_rebuild_on_preferences = ['all_checkboxes', 'labels', 'next_label', 'deadline_by_page']
+	_rebuild_on_preferences = ['all_checkboxes', 'labels', 'next_label', 'deadline_by_page', 'nonactionable_tags',
+				   'included_subtrees', 'excluded_subtrees' ]
 		# Rebuild database table if any of these preferences changed.
 		# But leave it alone if others change.
 
@@ -130,7 +140,11 @@ This is a core plugin shipping with zim.
 		self.task_label_re = None
 		self.next_label = None
 		self.next_label_re = None
+		self.nonactionable_tags = []
+		self.included_re = None
+		self.excluded_re = None
 		self.db_initialized = False
+		self._current_preferences = None
 
 	def initialize_ui(self, ui):
 		if ui.ui_type == 'gtk':
@@ -140,13 +154,15 @@ This is a core plugin shipping with zim.
 	def finalize_notebook(self, notebook):
 		# This is done regardsless of the ui type of the application
 		self.index = notebook.index
-		self.index.connect_after('initialize-db', self.initialize_db)
-		self.index.connect('page-indexed', self.index_page)
-		self.index.connect('page-deleted', self.remove_page)
+		self.connectto_all(self.index, (
+			('initialize-db', self.initialize_db, None, SIGNAL_AFTER),
+			('page-indexed', self.index_page),
+			('page-deleted', self.remove_page),
+		))
 		# We don't care about pages that are moved
 
 		db_version = self.index.properties['plugin_tasklist_format']
-		if db_version == SQL_FORMAT_VERSION_STRING:
+		if db_version == '%i.%i' % SQL_FORMAT_VERSION:
 			self.db_initialized = True
 
 		self._set_preferences()
@@ -154,10 +170,14 @@ This is a core plugin shipping with zim.
 	def initialize_db(self, index):
 		with index.db_commit:
 			index.db.executescript(SQL_CREATE_TABLES)
-		self.index.properties['plugin_tasklist_format'] = SQL_FORMAT_VERSION_STRING
+		self.index.properties['plugin_tasklist_format'] = '%i.%i' % SQL_FORMAT_VERSION
 		self.db_initialized = True
 
 	def do_preferences_changed(self):
+		if self._current_preferences is None \
+		or not self.db_initialized:
+			return
+
 		new_preferences = self._serialize_rebuild_on_preferences()
 		if new_preferences != self._current_preferences:
 			self._drop_table()
@@ -175,15 +195,40 @@ This is a core plugin shipping with zim.
 		if self.preferences['next_label']:
 			self.next_label = self.preferences['next_label']
 				# Adding this avoid the need for things like "TODO: Next: do this next"
-			self.next_label_re = re.compile(r'^' + re.escape(self.next_label) + r'(?!\w)' )
+			self.next_label_re = re.compile(r'^' + re.escape(self.next_label) + r':?\s+' )
 			self.task_labels.append(self.next_label)
 		else:
 			self.next_label = None
 			self.next_label_re = None
 
+		if self.preferences['nonactionable_tags']:
+			self.nonactionble_tags = [
+				t.strip().strip('@').lower()
+					for t in self.preferences['nonactionable_tags'].split(',')]
+		else:
+			self.nonactionble_tags = []
+
 		regex = r'^(' + '|'.join(map(re.escape, self.task_labels)) + r')(?!\w)'
 		self.task_label_re = re.compile(regex)
 
+		if self.preferences['included_subtrees']:
+			included = [i.strip().strip(':') for i in self.preferences['included_subtrees'].split(',')]
+			included.sort(key=lambda s: len(s), reverse=True) # longest first
+			included_re = '^(' + '|'.join(map(re.escape, included)) + ')(:.+)?$'
+			#~ print '>>>>>', "included_re", repr(included_re)
+			self.included_re = re.compile(included_re)
+		else:
+			self.included_re = None
+
+		if self.preferences['excluded_subtrees']:
+			excluded = [i.strip().strip(':') for i in self.preferences['excluded_subtrees'].split(',')]
+			excluded.sort(key=lambda s: len(s), reverse=True) # longest first
+			excluded_re = '^(' + '|'.join(map(re.escape, excluded)) + ')(:.+)?$'
+			#~ print '>>>>>', "excluded_re", repr(excluded_re)
+			self.excluded_re = re.compile(excluded_re)
+		else:
+			self.excluded_re = None
+
 
 	def _serialize_rebuild_on_preferences(self):
 		# string mapping settings that influence building the table
@@ -211,10 +256,35 @@ This is a core plugin shipping with zim.
 			except:
 				pass
 
+	def _excluded(self, path):
+		if self.included_re and self.excluded_re:
+			# judge which match is more specific
+			# this allows including subnamespace of excluded namespace
+			# and vice versa
+			inc_match = self.included_re.match(path.name)
+			exc_match = self.excluded_re.match(path.name)
+			if not exc_match:
+				return not bool(inc_match)
+			elif not inc_match:
+				return bool(exc_match)
+			else:
+				return len(inc_match.group(1)) < len(exc_match.group(1))
+		elif self.included_re:
+			return not bool(self.included_re.match(path.name))
+		elif self.excluded_re:
+			return bool(self.excluded_re.match(path.name))
+		else:
+			return False
+
 	def index_page(self, index, path, page):
 		if not self.db_initialized: return
 		#~ print '>>>>>', path, page, page.hascontent
+
 		tasksfound = self.remove_page(index, path, _emit=False)
+		if self._excluded(path):
+			if tasksfound:
+				self.emit('tasklist-changed')
+			return
 
 		parsetree = page.get_parsetree()
 		if not parsetree:
@@ -236,12 +306,16 @@ This is a core plugin shipping with zim.
 			deadline = dates[2]
 		else:
 			deadline = None
+
 		tasks = self._extract_tasks(parsetree, deadline)
+		#~ print 'TASKS', tasks
+
 		if tasks:
 			# Do insert with a single commit
 			with self.index.db_commit:
 				self._insert(path, 0, tasks)
 
+		if tasks or tasksfound:
 			self.emit('tasklist-changed')
 
 	def _insert(self, page, parentid, children):
@@ -249,9 +323,9 @@ This is a core plugin shipping with zim.
 		c = self.index.db.cursor()
 		for task, grandchildren in children:
 			c.execute(
-				'insert into tasklist(source, parent, haschildren, open, actionable, prio, due, description)'
-				'values (?, ?, ?, ?, ?, ?, ?, ?)',
-				(page.id, parentid, bool(grandchildren)) + task
+				'insert into tasklist(source, parent, haschildren, open, actionable, prio, due, tags, description)'
+				'values (?, ?, ?, ?, ?, ?, ?, ?, ?)',
+				(page.id, parentid, bool(grandchildren)) + tuple(task)
 			)
 			if grandchildren:
 				self._insert(page, c.lastrowid, grandchildren) # recurs
@@ -261,9 +335,21 @@ This is a core plugin shipping with zim.
 		@param parsetree: a L{zim.formats.ParseTree} object
 		@param defaultdate: default due date for the whole page (e.g. for calendar pages) as string
 		@returns: nested list of tasks, each task is given as a 2-tuple, 1st item is a tuple
-		with following properties: C{(open, actionable, prio, due, description)}, 2nd item
+		with following properties: C{(open, actionable, prio, due, tags, description)}, 2nd item
 		is a list of child tasks (if any).
 		'''
+		# Stack tuple indexes
+		LEVEL = 0
+		TASK = 1
+		CHILDREN = 2
+
+		# Task tuple indexes
+		OPEN = 0
+		ACT = 1
+		PRIO = 2
+		DATE = 3
+		TAGS = 4
+
 		tasks = []
 
 		for node in parsetree.findall('p'):
@@ -271,33 +357,27 @@ This is a core plugin shipping with zim.
 			# Check first line for task list header
 			istasklist = False
 			globaltags = []
+			globalactionable = True
+			globalprio = None
+			globaldate = defaultdate
 			if len(lines) >= 2 \
 			and isinstance(lines[0], basestring) \
 			and isinstance(lines[1], tuple) \
 			and self.task_labels and self.task_label_re.match(lines[0]):
-				for word in lines[0].strip(':').split()[1:]:
-					if word.startswith('@'):
-						globaltags.append(word)
-					else:
-						# not a header after all
-						globaltags = []
-						break
-				else:
-					# no break occurred - all OK
-					lines.pop(0)
-					istasklist = True
-
-			# Check line by line
-			LEVEL = 0
-			TASK = 1
-			CHILDREN = 2
+				# Parse the task list header as if it was a task and use it's
+				# attributes as defaults for the rest of the tasks in this block.
+				defaults = self._parse_task(lines[0])
+				defaults[TAGS] = defaults[TAGS].split(',')
+				globalactionable = defaults[ACT]
+				globalprio = defaults[PRIO]
+				globaltags.extend(defaults[TAGS])
+				globaldate = defaults[DATE]
+				lines.pop(0)
+				istasklist = True
 
 			stack = [] # stack of 3-tuples, (LEVEL, TASK, CHILDREN)
 
-
-			PRIO = 2
-			DATE = 3
-
+			# Check line by line
 			for item in lines:
 				if isinstance(item, tuple):
 					# checkbox or bullet
@@ -317,23 +397,36 @@ This is a core plugin shipping with zim.
 							if mydefaultdate == _NO_DATE:
 								mydefaultdate = defaultdate
 							mydefaultprio = stack[-1][TASK][PRIO]
+							mydefaultactionable = stack[-1][TASK][ACT]
+							inherited_tags = stack[-1][TASK][TAGS].split(',')
 						else:
-							mydefaultdate = defaultdate
-							mydefaultprio = None
+							mydefaultdate = globaldate
+							mydefaultprio = globalprio
+							mydefaultactionable = globalactionable
+							inherited_tags = globaltags
 
 						open = (bullet not in (CHECKED_BOX, XCHECKED_BOX))
 						if stack:
 							mytasks = stack[-1][CHILDREN]
 						else:
 							mytasks = tasks
-						task = self._parse_task(text, open=open, tags=globaltags, defaultdate=mydefaultdate, defaultprio=mydefaultprio, tasks=mytasks)
+						task = self._parse_task(text, open=open,
+								tags=inherited_tags,
+								actionable=mydefaultactionable,
+								defaultdate=mydefaultdate,
+								defaultprio=mydefaultprio,
+								tasks=mytasks)
 						children = []
 						if stack:
 							stack[-1][CHILDREN].append((task, children))
+							if task[OPEN]:
+								for parent in stack:
+									# child is open, so parent should be as well
+									parent[TASK][OPEN] = True
 						else:
 							tasks.append((task, children))
 
-						stack.append((list_level, task, children))
+						stack.append([list_level, task, children])
 					else:
 						# not a task - we already popped stack, now ignore text
 						pass
@@ -385,52 +478,53 @@ This is a core plugin shipping with zim.
 		return items
 
 	def _flatten(self, node):
-		# Just flatten everything to text
+		# Just flatten everything to text - but ignore strike out
 		text = node.text or ''
 		for child in node.getchildren():
+			if child.tag == 'strike':
+				continue # skip
 			text += self._flatten(child) # recurs
 			text += child.tail or ''
 		return text
 
-	def _parse_task(self, text, open=True, tags=None, defaultdate=None, defaultprio=None, tasks=None):
+	def _parse_task(self, text, open=True, tags=None, actionable=True, defaultdate=None, defaultprio=None, tasks=None):
+		## Note: do not modify text here, keep it as is, any cleanup is done by the widget
+
 		prio = text.count('!')
 		if defaultprio and prio == 0:
 			prio = defaultprio
 
-		global date # FIXME
-		date = _NO_DATE
+		if not tags:
+			tags = []
+		else:
+			# ensure we don't mutate the parent's tags list
+			tags = list(tags)
+		tags += _tag_re.findall(text)
 
-		def set_date(match):
-			global date
-			mydate = parse_date(match.group(0))
-			if mydate and date == _NO_DATE:
+		datematch = _date_re.search(text) # first match
+		if datematch:
+			mydate = parse_date(datematch.group(0))
+			if mydate:
 				date = '%04i-%02i-%02i' % mydate # (y, m, d)
-				#~ return match.group(0) # TEST
-				return ''
 			else:
-				# No match or we already had a date
-				return match.group(0)
-
-		if tags:
-			for tag in tags:
-				if not tag in text:
-					text += ' ' + tag
-
-		text = date_re.sub(set_date, text)
+				date = _NO_DATE
+		else:
+			date = _NO_DATE
 
 		if defaultdate and date == _NO_DATE:
 			date = defaultdate
 
-		if self.next_label_re.match(text):
-			if tasks and tasks[-1][0][0]: # previous task still open
+		if actionable:
+			if any(t.lower().strip('@') in self.nonactionble_tags for t in tags):
 				actionable = False
-			else:
-				actionable = True
-		else:
-			actionable = True
+			elif self.next_label_re.match(text):
+				if tasks and tasks[-1][0][0]: # previous task still open
+					actionable = False
+		# else parent was non-actionable already, so stay non-actionable
 
-		return (open, actionable, prio, date, text)
-			# (open, actionable, prio, due, description)
+		tags = ','.join(t.strip('@') for t in tags)
+		return [open, actionable, prio, date, tags, text]
+			# (open, actionable, prio, due, tags, description)
 
 
 	def remove_page(self, index, path, _emit=True):
@@ -519,7 +613,7 @@ class TaskListDialog(Dialog):
 			# we'll end with a too small dialog and no way to resize it
 		hbox = gtk.HBox(spacing=5)
 		self.vbox.pack_start(hbox, False)
-		self.hpane = gtk.HPaned()
+		self.hpane = HPaned()
 		self.uistate.setdefault('hpane_pos', 75)
 		self.hpane.set_position(self.uistate['hpane_pos'])
 		self.vbox.add(self.hpane)
@@ -528,19 +622,11 @@ class TaskListDialog(Dialog):
 		self.uistate.setdefault('only_show_act', False)
 		self.task_list = TaskListTreeView(self.ui, plugin, filter_actionable=self.uistate['only_show_act'])
 		self.task_list.set_headers_visible(True) # Fix for maemo
-		scrollwindow = gtk.ScrolledWindow()
-		scrollwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-		scrollwindow.set_shadow_type(gtk.SHADOW_IN)
-		scrollwindow.add(self.task_list)
-		self.hpane.add2(scrollwindow)
+		self.hpane.add2(ScrolledWindow(self.task_list))
 
 		# Tag list
 		self.tag_list = TagListTreeView(self.task_list)
-		scrollwindow = gtk.ScrolledWindow()
-		scrollwindow.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
-		scrollwindow.set_shadow_type(gtk.SHADOW_IN)
-		scrollwindow.add(self.tag_list)
-		self.hpane.add1(scrollwindow)
+		self.hpane.add1(ScrolledWindow(self.tag_list))
 
 		# Filter input
 		hbox.pack_start(gtk.Label(_('Filter')+': '), False) # T: Input label
@@ -573,16 +659,35 @@ class TaskListDialog(Dialog):
 		self.statistics_label = gtk.Label()
 		hbox.pack_end(self.statistics_label, False)
 
-		def set_statistics(o):
+
+		def set_statistics():
 			total, stats = self.task_list.get_statistics()
 			text = ngettext('%i open item', '%i open items', total) % total
 				# T: Label for statistics in Task List, %i is the number of tasks
 			text += ' (' + '/'.join(map(str, stats)) + ')'
 			self.statistics_label.set_text(text)
 
-		set_statistics(self.task_list)
-		self.plugin.connect('tasklist-changed', set_statistics)
-			# Make sure this is connected after the task list connected to same signal
+		set_statistics()
+
+		def on_tasklist_changed(o):
+			self.task_list.refresh()
+			self.tag_list.refresh(self.task_list)
+			set_statistics()
+
+		callback = DelayedCallback(10, on_tasklist_changed)
+			# Don't really care about the delay, but want to
+			# make it less blocking - should be async preferably
+			# now it is at least on idle
+		self.connectto(plugin, 'tasklist-changed', callback)
+
+		# Async solution fall because sqlite not multi-threading
+		# (see also todo item for async in DelayedSignal class)
+
+		#~ def async_call(o):
+			#~ from zim.async import AsyncOperation
+			#~ op = AsyncOperation(on_tasklist_changed, args=(o,))
+			#~ op.start()
+		#~ self.connectto(plugin, 'tasklist-changed', async_call)
 
 	def do_response(self, response):
 		self.uistate['hpane_pos'] = self.hpane.get_position()
@@ -599,6 +704,7 @@ class TagListTreeView(SingleClickTreeView):
 	_type_separator = 0
 	_type_label = 1
 	_type_tag = 2
+	_type_untagged = 3
 
 	def __init__(self, task_list):
 		model = gtk.ListStore(str, int, int, int) # tag name, number of tasks, type, weight
@@ -625,8 +731,6 @@ class TagListTreeView(SingleClickTreeView):
 		self.get_selection().connect('changed', self.on_selection_changed)
 
 		self.refresh(task_list)
-		task_list.plugin.connect('tasklist-changed', lambda o: self.refresh(task_list))
-			# Make sure this is connected after the task list connected to same signal
 
 	def get_tags(self):
 		'''Returns current selected tags, or None for all tags'''
@@ -634,6 +738,8 @@ class TagListTreeView(SingleClickTreeView):
 		for row in self._get_selected():
 			if row[2] == self._type_tag:
 				tags.append(row[0])
+			elif row[2] == self._type_untagged:
+				tags.append(_NO_TAGS)
 		return tags or None
 
 	def get_labels(self):
@@ -673,9 +779,14 @@ class TagListTreeView(SingleClickTreeView):
 			if label in labels and label != plugin.next_label:
 				model.append((label, labels[label], self._type_label, pango.WEIGHT_BOLD))
 
+		tags = self.task_list.get_tags()
+		if _NO_TAGS in tags:
+			n_untagged = tags.pop(_NO_TAGS)
+			model.append((_('Untagged'), n_untagged, self._type_untagged, pango.WEIGHT_NORMAL))
+			# T: label in tasklist plugins for tasks without a tag
+
 		model.append(('', 0, self._type_separator, 0)) # separator
 
-		tags = self.task_list.get_tags()
 		for tag in natural_sorted(tags):
 			model.append((tag, tags[tag], self._type_tag, pango.WEIGHT_NORMAL))
 
@@ -694,8 +805,7 @@ class TagListTreeView(SingleClickTreeView):
 		if not self._block_selection_change:
 			tags = self.get_tags()
 			labels = self.get_labels()
-			self.task_list.set_tag_filter(tags)
-			self.task_list.set_label_filter(labels)
+			self.task_list.set_tag_filter(tags, labels)
 
 
 HIGH_COLOR = '#EF5151' # red (derived from Tango style guide - #EF2929)
@@ -714,10 +824,11 @@ class TaskListTreeView(BrowserTreeView):
 	ACT_COL = 5 # actionable
 	OPEN_COL = 6 # item not closed
 	TASKID_COL = 7
+	TAGS_COL = 8
 
 	def __init__(self, ui, plugin, filter_actionable):
-		self.real_model = gtk.TreeStore(bool, int, str, str, str, bool, bool, int)
-			# VIS_COL, PRIO_COL, TASK_COL, DATE_COL, PAGE_COL, ACT_COL, OPEN_COL, TASKID_COL
+		self.real_model = gtk.TreeStore(bool, int, str, str, str, bool, bool, int, object)
+			# VIS_COL, PRIO_COL, TASK_COL, DATE_COL, PAGE_COL, ACT_COL, OPEN_COL, TASKID_COL, TAGS_COL
 		model = self.real_model.filter_new()
 		model.set_visible_column(self.VIS_COL)
 		model = gtk.TreeModelSort(model)
@@ -769,9 +880,20 @@ class TaskListTreeView(BrowserTreeView):
 			self.set_tooltip_column(self.TASK_COL)
 
 		# Rendering of the Date column
+		use_workweek = plugin.preferences['use_workweek']
+		day_of_week = datetime.date.today().isoweekday()
+		if use_workweek and day_of_week == 4:
+			# Today is Thursday - 2nd day ahead is after the weekend
+			delta1, delta2 = 1, 3
+		elif use_workweek and day_of_week == 5:
+			# Today is Friday - next day ahead is after the weekend
+			delta1, delta2 = 3, 4
+		else:
+			delta1, delta2 = 1, 2
+
 		today    = str( datetime.date.today() )
-		tomorrow = str( datetime.date.today() + datetime.timedelta(days=1))
-		dayafter = str( datetime.date.today() + datetime.timedelta(days=2))
+		tomorrow = str( datetime.date.today() + datetime.timedelta(days=delta1))
+		dayafter = str( datetime.date.today() + datetime.timedelta(days=delta2))
 		def render_date(col, cell, model, i):
 			date = model.get_value(i, self.DATE_COL)
 			if date == _NO_DATE:
@@ -781,8 +903,9 @@ class TaskListTreeView(BrowserTreeView):
 				# TODO allow strftime here
 
 			if date <= today: color = HIGH_COLOR
-			elif date == tomorrow: color = MEDIUM_COLOR
-			elif date == dayafter: color = ALERT_COLOR
+			elif date <= tomorrow: color = MEDIUM_COLOR
+			elif date <= dayafter: color = ALERT_COLOR
+				# "<=" because tomorrow and/or dayafter can be after the weekend
 			else: color = None
 			cell.set_property('cell-background', color)
 
@@ -802,7 +925,6 @@ class TaskListTreeView(BrowserTreeView):
 
 		# Finalize
 		self.refresh()
-		self.plugin.connect_object('tasklist-changed', self.__class__.refresh, self)
 
 		# HACK because we can not register ourselves :S
 		self.connect('row_activated', self.__class__.do_row_activated)
@@ -837,48 +959,55 @@ class TaskListTreeView(BrowserTreeView):
 		rows = list(self.plugin.list_tasks(task))
 
 		# First cache + sort tasks to ensure stability of the list
-		for row in rows:
+		for i, row in enumerate(rows):
 			if not row['source'] in path_cache:
 				path = self.plugin.get_path(row)
 				if path is None:
 					# Be robust for glitches - filter these out
-					row['source'] = None
+					rows[i] = None
 				else:
 					path_cache[row['source']] = path
 
-		rows = [r for r in rows if r['source'] is not None] # filter out missing paths
+		rows = [r for r in rows if r is not None] # filter out missing paths
 
 		rows.sort(key=lambda r: path_cache[r['source']].name)
 
-		# Count them (note: these matches fail after formatting applied)
+		# Then format them and add them to the model
 		for row in rows:
 			if not row['open']:
-				continue # Only count open items
+				continue # Only include open items for now
+			path = path_cache[row['source']]
 
+			# Update labels
 			for label in self.plugin.task_label_re.findall(row['description']):
 				self._labels[label] = self._labels.get(label, 0) + 1
 
-			for tag in tag_re.findall(row['description']):
-				self._tags[tag] = self._tags.get(tag, 0) + 1
-
+			# Update tag count
+			tags = row['tags'].split(',')
 			if self.plugin.preferences['tag_by_page']:
-				path = path_cache[row['source']]
-				for part in path.parts:
-					self._tags[part] = self._tags.get(part, 0) + 1
+				tags = tags + path.parts
 
-		# Then format them and add them to the model
-		for row in rows:
-			path = path_cache[row['source']]
-			task = encode_markup_text(row['description'])
+			if tags:
+				for tag in tags:
+					self._tags[tag] = self._tags.get(tag, 0) + 1
+			else:
+				self._tags[_NO_TAGS] = self._tags.get(_NO_TAGS, 0) + 1
+
+
+			# Format description
+			task = _date_re.sub('', row['description'], count=1)
+			task = encode_markup_text(task)
 			task = re.sub('\s*!+\s*', ' ', task) # get rid of exclamation marks
 			task = self.plugin.next_label_re.sub('', task) # get rid of "next" label in description
 			if row['actionable']:
-				task = tag_re.sub(r'<span color="#ce5c00">@\1</span>', task) # highlight tags - same color as used in pageview
+				task = _tag_re.sub(r'<span color="#ce5c00">@\1</span>', task) # highlight tags - same color as used in pageview
 				task = self.plugin.task_label_re.sub(r'<b>\1</b>', task) # highlight labels
 			else:
 				task = r'<span color="darkgrey">%s</span>' % task
-			modelrow = [False, row['prio'], task, row['due'], path.name, row['actionable'], row['open'], row['id']]
-				# VIS_COL, PRIO_COL, TASK_COL, DATE_COL, PAGE_COL, ACT_COL, OPEN_COL, TASKID_COL
+
+			# Insert all columns
+			modelrow = [False, row['prio'], task, row['due'], path.name, row['actionable'], row['open'], row['id'], tags]
+				# VIS_COL, PRIO_COL, TASK_COL, DATE_COL, PAGE_COL, ACT_COL, OPEN_COL, TASKID_COL, TAGS_COL
 			modelrow[0] = self._filter_item(modelrow)
 			myiter = self.real_model.append(iter, modelrow)
 
@@ -953,18 +1082,17 @@ class TaskListTreeView(BrowserTreeView):
 		else:
 			return 0, []
 
-	def set_tag_filter(self, tags):
+	def set_tag_filter(self, tags=None, labels=None):
 		if tags:
 			self.tag_filter = [tag.lower() for tag in tags]
 		else:
 			self.tag_filter = None
-		self._eval_filter()
 
-	def set_label_filter(self, labels):
 		if labels:
 			self.label_filter = [label.lower() for label in labels]
 		else:
 			self.label_filter = None
+
 		self._eval_filter()
 
 	def _eval_filter(self):
@@ -993,6 +1121,7 @@ class TaskListTreeView(BrowserTreeView):
 
 		description = modelrow[self.TASK_COL].decode('utf-8').lower()
 		pagename = modelrow[self.PAGE_COL].decode('utf-8').lower()
+		tags = [t.lower() for t in modelrow[self.TAGS_COL]]
 
 		if visible and self.label_filter:
 			# Any labels need to be present
@@ -1003,17 +1132,12 @@ class TaskListTreeView(BrowserTreeView):
 				visible = False # no label found
 
 		if visible and self.tag_filter:
-			# And any tag should match (or pagename if tag_by_page)
-			for tag in self.tag_filter:
-				if self.plugin.preferences['tag_by_page']:
-					if '@'+tag in description \
-					or tag in pagename.split(':'):
-						break # keep visible True
-				else:
-					if '@'+tag in description:
-						break # keep visible True
+			# Any tag should match
+			if (_NO_TAGS in self.tag_filter and not tags) \
+			or any(tag in tags for tag in self.tag_filter):
+				visible = True
 			else:
-				visible = False # no tag found
+				visible = False
 
 		if visible and self.filter:
 			# And finally the filter string should match
@@ -1039,11 +1163,11 @@ class TaskListTreeView(BrowserTreeView):
 
 	def do_initialize_popup(self, menu):
 		item = gtk.ImageMenuItem('gtk-copy')
-		item.connect_object('activate', self.__class__.copy_to_clipboard, self)
+		item.connect('activate', self.copy_to_clipboard)
 		menu.append(item)
 		self.populate_popup_expand_collapse(menu)
 
-	def copy_to_clipboard(self):
+	def copy_to_clipboard(self, *a):
 		'''Exports currently visible elements from the tasks list'''
 		logger.debug('Exporting to clipboard current view of task list.')
 		text = self.get_visible_data_as_csv()
