@@ -6,9 +6,9 @@
 This module contains the logic for searching in a notebook.
 
 Supported operators:
- 	- "NOT", "not" and "-"
- 	- "AND", "and", "+" and "&&"
- 	- "OR", "or" and "||"
+	- "NOT", "not" and "-"
+	- "AND", "and", "+" and "&&"
+	- "OR", "or" and "||"
 
 Order of precedence: AND, OR, NOT
 so "foo AND NOT bar OR baz" means AND(foo, OR(NOT(bar), baz))
@@ -44,11 +44,14 @@ For content '*' can occur on both sides, but does not match whitespace
 
 
 import re
+import logging
 
 from zim.parsing import split_quoted_strings, unescape_quoted_string, Re
 from zim.notebook import Path, PageNameError
 from zim.index import LINK_DIR_BACKWARD, LINK_DIR_FORWARD
 
+logger = logging.getLogger('zim.search')
+
 
 OPERATOR_OR = 1
 OPERATOR_AND = 2
@@ -141,11 +144,12 @@ class Query(object):
 				if keyword == 'links':
 					keyword = 'linksfrom'
 				tokens.append(QueryTerm(keyword, string))
-			elif tag_re.match(w):
-				tokens.append(QueryTerm('tag', w[1:]))
 			else:
 				w = unescape_quoted_string(w)
-				tokens.append(QueryTerm('contentorname', w)) # default keyword
+				if tag_re.match(w):
+					tokens.append(QueryTerm('tag', w[1:]))
+				else:
+					tokens.append(QueryTerm('contentorname', w)) # default keyword
 		#~ print tokens
 
 		# Then parse NOT operator out
@@ -258,12 +262,12 @@ class SearchSelection(PageSelection):
 		# it is recursive for nested QueryGroup objects and calls
 		# _process_from_index and _process_content to handle
 		# QueryTerms in the group. It takes care of combining the
-		# results from various terms and calling the callback 
+		# results from various terms and calling the callback
 		# function when possible
 
 		# Special case to optimize for simple OR query to give callback results
 		if len(group) == 1 and isinstance(group[0], QueryGroup):
-			group = group[0] 
+			group = group[0]
 
 		# For optimization we sort the terms in the group based  on how
 		# easy we can get them. Anything that needs content is last.
@@ -451,7 +455,9 @@ class SearchSelection(PageSelection):
 				scope = set()
 				for p in index.walk():
 					scope.add(p)
-			myresults = scope - myresults
+			inverse = scope - myresults
+			myresults.clear()
+			myresults.update(inverse)
 
 		for path in myresults:
 			self._count_score(path, scoring)
@@ -467,8 +473,8 @@ class SearchSelection(PageSelection):
 		# Note that this rationale is for flat searches, once sub-groups
 		# are involved things get less optimized.
 		#
-		# For AND 'scope' will be the results of previous steps, we make a subset 
-		# of this. In 'results' will only be any final results already obtained from 
+		# For AND 'scope' will be the results of previous steps, we make a subset
+		# of this. In 'results' will only be any final results already obtained from
 		# contentorname optimization
 		# For OR 'results' is whatever was found so far while 'scope' can be larger
 		# we extend the results with any matches from scope
@@ -486,7 +492,12 @@ class SearchSelection(PageSelection):
 
 		for page in generator:
 			#~ print '!! Search content', page
-			tree = page.get_parsetree()
+			try:
+				tree = page.get_parsetree()
+			except:
+				logger.exception('Exception while reading: %s', page)
+				continue
+
 			if tree is None:
 				continue # Assume need to have content even for negative query
 
