@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2008 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2008-2013 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''This module contains classes for template processing.
 
@@ -70,7 +70,7 @@ import zim
 import zim.formats
 import zim.datetimetz as datetime
 from zim.errors import Error
-from zim.fs import File
+from zim.fs import File, Dir, format_file_size
 from zim.config import data_dirs
 from zim.parsing import Re, TextBuffer, split_quoted_strings, unescape_quoted_string, is_path_re
 from zim.formats import ParseTree, Element, TreeBuilder
@@ -85,34 +85,54 @@ __all__ = [
 ]
 
 
-def list_templates(format):
-	'''Returns a dict mapping template names to file paths.'''
-	format = format.lower()
-	templates = {}
-	path = list(data_dirs(('templates', format)))
+def list_template_categories():
+	'''Returns a list of categories (sub folders)'''
+	dirs = data_dirs('templates')
+	categories = set()
+	for dir in dirs:
+		for name in dir.list():
+			## TODO list_objects would help here + a filter like type=Dir
+			if dir.subdir(name).isdir():
+				categories.add(name)
+
+	return sorted(categories)
+
+
+def list_templates(category):
+	'''Returns a list of template names
+	@param category: a category (sub folder) with tempaltes, e.g. "html"
+	@returns: a list of 2-tuples of the template names and the file
+	basename for the template file
+	'''
+	category = category.lower()
+	templates = set()
+	path = list(data_dirs(('templates', category)))
 	path.reverse()
 	for dir in path:
-		for file in dir.list():
-			i = file.rfind('.') # match begin of file extension
-			if i >= 0:
-				if '~' in file[i:]:
-					continue # Ignore tmp files etc.
-				#~ templates[file[0:i]] = dir.file(file) FIXME
-				import os
-				templates[file[0:i]] = os.path.join(dir.path, file)
-	return templates
+		for basename in dir.list():
+			name = basename.rsplit('.', 1)[0] # robust if no '.' in basename
+			templates.add((name, basename))
+	return sorted(templates)
 
 
 def get_template(format, template):
 	'''Returns a Template object for a template name, file path, or File object'''
+	# NOTE: here the "category" needs to be a format at the same time !
 	if isinstance(template, File):
 		file = template
 	else:
 		if not is_path_re.match(template):
-			try:
-				templates = list_templates(format)
-				file = File(templates[template])
-			except KeyError:
+			file = None
+			path = list(data_dirs(('templates', format)))
+			path.reverse()
+			for dir in path:
+				for basename in dir.list():
+					name = basename.rsplit('.')[0] # robust if no '.' in basename
+					if name == template:
+						file = dir.file(basename)
+						break
+
+			if not file:
 				file = File(template)
 		else:
 			file = File(template)
@@ -128,34 +148,36 @@ def get_template(format, template):
 
 
 class TemplateError(Error):
-	pass
+
+	msg = 'Template Error'
+
+	def __init__(self, token, error):
+		self.description = self._description.strip() + '\n\n'
+		if token.text:
+			self.description += "Error in \"%s\" line \"%i\" at \"%s\":" % (token.file, token.lineno, token.text)
+		else:
+			self.description += "Error in \"%s\" line \"%i\":" % (token.file, token.lineno)
+		self.description += '\n\n' + error.strip()
 
 
 class TemplateSyntaxError(TemplateError):
 
-	description = '''\
+	msg = 'Template Syntax Error'
+
+	_description = '''\
 An error occurred while parsing a template.
 It seems the template contains some invalid syntax.
 '''
 
 
-	def __init__(self, msg):
-		self._msg = msg
-		self.file = '<unknown file>'
-		self.line = 0
-
-	@property
-	def msg(self):
-		return 'Syntax error at "%s" line %i: %s' % \
-						(self.file, self.line, self.msg)
-
-
 class TemplateProcessError(TemplateError):
 
-	description = '''
+	msg = 'Template Processing Error'
+
+	_description = '''\
 A run-time error occurred while processing a template.
-This can be due to the template calling functions that are
-not available, or it can be a glitch in the program.
+This can be e.g. due to the template calling functions that
+are not available, or it can be a glitch in the program.
 '''
 
 
@@ -192,8 +214,9 @@ class GenericTemplate(object):
 		'''Constructor takes a file path or an opened file handle'''
 		if isinstance(input, basestring):
 			input = input.splitlines(True)
-		self.name = name or '<open file>'
+		self.file = name or '<open file>'
 		self.lineno = 0
+		self.text = None # needs to be set to None for use with TemplateError()
 		self.tokens = TemplateTokenList()
 		self.stack = [ self.tokens ]
 		self._parse(input)
@@ -226,7 +249,7 @@ class GenericTemplate(object):
 			#~ print "TOKEN >>>%s<<<" % string
 
 			if string.startswith('IF'):
-				token = IFToken(string[2:])
+				token = IFToken(self.file, self.lineno, string[2:])
 				self.stack[-1].append(token)
 				self.stack.append(token.blocks[0])
 			elif string.startswith('ELSIF'):
@@ -235,7 +258,7 @@ class GenericTemplate(object):
 					iftoken = self.stack[-1][-1]
 					assert isinstance(iftoken, IFToken)
 				except:
-					raise TemplateSyntaxError, 'ELSIF outside IF block'
+					raise TemplateSyntaxError(self, 'ELSIF outside IF block')
 				block = iftoken.add_block(string[5:])
 				self.stack.append(block)
 			elif string.startswith('ELSE'):
@@ -244,57 +267,52 @@ class GenericTemplate(object):
 					iftoken = self.stack[-1][-1]
 					assert isinstance(iftoken, IFToken)
 				except:
-					raise TemplateSyntaxError, 'ELSE outside IF block'
+					raise TemplateSyntaxError(self, 'ELSE outside IF block')
 				block = iftoken.add_block()
 				self.stack.append(block)
 			elif string.startswith('FOREACH'):
-				token = FOREACHToken(string[7:])
+				token = FOREACHToken(self.file, self.lineno, string[7:])
 				self.stack[-1].append(token)
 				self.stack.append(token.foreach_block)
 			elif string.startswith('END'):
 				if not len(self.stack) > 1:
-					raise TemplateSyntaxError, 'END outside block'
+					raise TemplateSyntaxError(self, 'END outside block')
 				self.stack.pop()
 			else:
 				if string.startswith('SET'):
-					token = SETToken(string[3:])
+					token = SETToken(self.file, self.lineno, string[3:])
 				elif string.startswith('GET'):
-					token = GETToken(string[3:])
+					token = GETToken(self.file, self.lineno, string[3:])
 				elif self._token_re.match(string):
-					raise TemplateSyntaxError, 'Unknown directive: %s' % self._token_re[1]
+					raise TemplateSyntaxError(self, 'Unknown directive: %s' % self._token_re[1])
 				elif string.find('=') >= 0:	# implicit SET
-					token = SETToken(string)
+					token = SETToken(self.file, self.lineno, string)
 				else:  # implicit GET
-					token = GETToken(string)
+					token = GETToken(self.file, self.lineno, string)
 				self.stack[-1].append(token)
 
-		try:
-			for line in input:
-				self.lineno += 1
+		for line in input:
+			self.lineno += 1
 
-				while line.find('[%') >= 0:
-					(pre, sep, line) = line.partition('[%')
-					(cmd, sep, line) = line.partition('%]')
-					if not sep:
-						raise TemplateSyntaxError, "unmatched '[%'"
-					append_text(pre)
-					if cmd.startswith('-'): # '[%-'
-						try:
-							if isinstance(self.stack[-1][-1], basestring):
-								self.stack[-1][-1] = self.stack[-1][-1].rstrip()
-						except IndexError:
-							pass
-					if cmd.endswith('-'): # '-%]'
-						line = line.lstrip()
-					cmd = cmd.strip('-')
-					append_token(cmd)
-				append_text(line)
-			if len(self.stack) > 1:
-				raise TemplateSyntaxError, 'open block at end of input - forgot END ?'
-		except TemplateSyntaxError, error:
-			error.file = self.name
-			error.line = self.lineno
-			raise error
+			while line.find('[%') >= 0:
+				(pre, sep, line) = line.partition('[%')
+				(cmd, sep, line) = line.partition('%]')
+				if not sep:
+					raise TemplateSyntaxError(self, "unmatched '[%'")
+				append_text(pre)
+				if cmd.startswith('-'): # '[%-'
+					try:
+						if isinstance(self.stack[-1][-1], basestring):
+							self.stack[-1][-1] = self.stack[-1][-1].rstrip()
+					except IndexError:
+						pass
+				if cmd.endswith('-'): # '-%]'
+					line = line.lstrip()
+				cmd = cmd.strip('-')
+				append_token(cmd)
+			append_text(line)
+		if len(self.stack) > 1:
+			raise TemplateSyntaxError(self, 'open block at end of input - forgot END ?')
 
 
 class Template(GenericTemplate):
@@ -310,6 +328,7 @@ class Template(GenericTemplate):
 		else:
 			self.resources_dir = resources_dir
 		GenericTemplate.__init__(self, input, name)
+		self.template_options = None
 
 	def set_linker(self, linker):
 		self.linker = linker
@@ -321,7 +340,7 @@ class Template(GenericTemplate):
 		The attribute 'pages' can be used to supply page objects for
 		special pages, like 'next', 'previous', 'index' and 'home'.
 		'''
-		options = {} # this dict is accessible from the template and is
+		self.template_options = {} # this dict is writable from the template and is
 		             # passed on to the format
 
 		if pages:
@@ -331,7 +350,7 @@ class Template(GenericTemplate):
 				if not mypages[key] is None:
 					pages[key] = PageProxy(
 						notebook, mypages[key],
-						self.format, self.linker, options)
+						self.format, self.linker, self.template_options)
 		else:
 			pages = {}
 
@@ -343,13 +362,14 @@ class Template(GenericTemplate):
 			},
 			'page': PageProxy(
 				notebook, page,
-				self.format, self.linker, options),
+				self.format, self.linker, self.template_options),
 			'pages': pages,
 			'strftime': StrftimeFunction(),
+			'strfcal': StrfcalFunction(),
 			'url': TemplateFunction(self.url),
 			'resource': TemplateFunction(self.resource_url),
-			'pageindex' : PageIndexFunction(notebook, page, self.format, self.linker, options),
-			'options': options,
+			'pageindex' : PageIndexFunction(notebook, page, self.format, self.linker, self.template_options),
+			'options': self.template_options,
 			# helpers that allow to write TRUE instead of 'TRUE' in template functions
 			'TRUE' : 'True', 'FALSE' : 'False',
 		}
@@ -360,6 +380,13 @@ class Template(GenericTemplate):
 			# first part of the template
 
 		TemplateManager.emit('process-page', self, page, dict)
+
+		# Bootstrap options as user modifiable part of dictionary
+		# need to assign in TempalteDict, so it goes in _user
+		dict = TemplateDict(dict)
+		dict[TemplateParam('options')] = self.template_options
+
+		# Finally process the template
 		output = GenericTemplate.process(self, dict)
 
 		# Caching last processed dict because any pages in the dict
@@ -379,8 +406,13 @@ class Template(GenericTemplate):
 		'''Static method callable from the template, returns a string'''
 		if link is None:
 			return ''
-		elif not isinstance(link, basestring):
-			link = ':' + link.name # special page link index ?
+		elif isinstance(link, FilePathProxy):
+			link = link.path
+		elif isinstance(link, (File, Dir)):
+			link = link.uri
+		elif not isinstance(link, basestring): # Path, PageProxy
+			link = ':' + link.name
+		# else basestring
 
 		linker = dict[TemplateParam('page')]._linker # bit of a hack
 		if linker:
@@ -417,6 +449,21 @@ class TemplateTokenList(list):
 
 class TemplateToken(object):
 
+	def __init__(self, file, line, text=None):
+		self.file = file
+		self.lineno = line
+		if text is not None:
+			self.text = text.strip()
+			try:
+				self.parse_string(self.text)
+			except Error, error:
+				raise TemplateSyntaxError(self, error.msg)
+		else:
+			self.text = None
+
+	def parse_string(self, string):
+		raise NotImplemented, 'Class %s does not take string' % self.__class__.__name__
+
 	def parse_expr(self, string):
 		'''This method parses an expression and returns an object of either
 		class TemplateParam, TemplateParamList or TemplateFuntionParam.
@@ -433,7 +480,7 @@ class TemplateToken(object):
 				split_quoted_strings(string, unescape=False) ):
 				if i % 2:
 					if w != ',':
-						raise TemplateSyntaxError, string
+						raise TemplateSyntaxError(self, string)
 				elif w.startswith('"') or w.startswith("'"):
 					list.append(TemplateLiteral(unescape_quoted_string(w)))
 				else:
@@ -455,13 +502,29 @@ class TemplateToken(object):
 		else:
 			return TemplateParam(string)
 
+	def process(self, dict):
+		try:
+			re = self.do_process(dict)
+		except TemplateError:
+			raise
+		except Error, error:
+			raise TemplateProcessError(self, error.msg)
+		except Exception:
+			import traceback
+			trace = traceback.format_exc()
+			raise TemplateProcessError(self, trace)
+		return re
+
+	def do_process(self, dict):
+		raise NotImplemented
+
 
 class GETToken(TemplateToken):
 
-	def __init__(self, string):
+	def parse_string(self, string):
 		self.expr = self.parse_expr(string)
 
-	def process(self, dict):
+	def do_process(self, dict):
 		value = self.expr.evaluate(dict)
 		if value:
 			return [value]
@@ -471,21 +534,21 @@ class GETToken(TemplateToken):
 
 class SETToken(TemplateToken):
 
-	def __init__(self, string):
+	def parse_string(self, string):
 		(var, sep, val) = string.partition('=')
 		if not sep:
-			raise TemplateSyntaxError, string
+			raise TemplateSyntaxError(self, string)
 		self.param = TemplateParam(var.strip())
 		self.expr = self.parse_expr(val)
 
-	def process(self, dict):
+	def do_process(self, dict):
 		dict[self.param] = self.expr.evaluate(dict)
 		return []
 
 
 class IFToken(TemplateToken):
 
-	def __init__(self, string):
+	def parse_string(self, string):
 		self.blocks = []
 		self.add_block(string)
 
@@ -507,7 +570,7 @@ class IFToken(TemplateToken):
 
 		return self.blocks[-1]
 
-	def process(self, dict):
+	def do_process(self, dict):
 		for block in self.blocks:
 			if block.if_statement is None:
 				# ELSE block
@@ -533,17 +596,17 @@ class IFToken(TemplateToken):
 
 class FOREACHToken(TemplateToken):
 
-	def __init__(self, string):
+	def parse_string(self, string):
 		(var, sep, val) = string.partition('=')
 		if not sep:
 			(var, sep, val) = string.partition('IN')
 		if not sep:
-			raise TemplateSyntaxError, string
+			raise TemplateSyntaxError(self, string)
 		self.param = TemplateParam(var.strip())
 		self.expr = self.parse_expr(val)
 		self.foreach_block = TemplateTokenList()
 
-	def process(self, dict):
+	def do_process(self, dict):
 		values = self.expr.evaluate(dict)
 		# FIXME how to check if values is iterable ?
 		output = []
@@ -578,11 +641,28 @@ class TemplateParam(object):
 	def __init__(self, name):
 		self.name = name
 		parts = name.split('.')
+		key = parts.pop()
+		if '[' in key:
+			i = key.index('[')
+			parts.append(key[:i])
+			key = key[i:]
+
 		for n in parts:
 			if not self._param_re.match(n):
-				raise TemplateSyntaxError, 'invalid parameter: %s' % name
-		self.path = parts[:-1]
-		self.key = parts[-1]
+				raise Error, 'invalid parameter: %s' % name
+
+		# support param["foo"] syntax for dicts
+		if parts and key.startswith('[') and key.endswith(']'):
+			key = unescape_quoted_string(key[1:-1])
+			if key.startswith('_'):
+				raise TemplateSyntaxError(self, 'invalid dictionary key in: %s' % name)
+		elif not self._param_re.match(key):
+			raise Error, 'invalid parameter: %s' % name
+		else:
+			pass
+
+		self.path = parts
+		self.key = key
 
 	def __str__(self):
 		return self.name
@@ -605,7 +685,7 @@ class TemplateFunctionParam(TemplateParam):
 		func = dict[self]
 		args = self.args.evaluate(dict)
 		if not isinstance(func, TemplateFunction):
-			raise TemplateProcessError, 'No such function: %s' % self.name
+			raise Error, 'No such function: %s' % self.name
 		return func(dict, *args)
 
 
@@ -631,10 +711,17 @@ class TemplateFunction(object):
 	'''
 
 	def __init__(self, function):
+		'''Constructor. Base class takes a regular function as argument
+		and wraps it to beccome a template functions.
+		'''
 		self.function = function
 
-	def __call__(self, *args):
-		return self.function(*args)
+	def __call__(self, dict, *args):
+		'''Execute the function
+		@param dict: the L{TemplateDict} for the page being processed
+		@param args: the arguments supplied in the template
+		'''
+		return self.function(dict, *args)
 
 
 class StrftimeFunction(TemplateFunction):
@@ -651,7 +738,7 @@ class StrftimeFunction(TemplateFunction):
 			elif isinstance(date, (datetime.date, datetime.datetime)):
 				string = date.strftime(format)
 			else:
-				raise AssertionError, 'Not a datetime object: %s', date
+				raise Error, 'Not a datetime object: %s' % date
 			return string.decode(locale.getpreferredencoding())
 				# strftime returns locale as understood by the C api
 				# unfortunately there is no guarantee we can actually
@@ -660,6 +747,24 @@ class StrftimeFunction(TemplateFunction):
 			logger.exception('Error in strftime "%s"', format)
 
 
+class StrfcalFunction(TemplateFunction):
+	'''Template function wrapper for strfcal'''
+
+	def __init__(self):
+		pass
+
+	def __call__(self, dict, format, date=None):
+		format = str(format) # Needed to please datetime.strftime()
+		try:
+			if date is None:
+				date = datetime.now()
+
+			string = datetime.strfcal(format, date)
+		except:
+			logger.exception('Error in strftime "%s"', format)
+
+
+
 class PageIndexFunction(TemplateFunction):
 	'''Template function to build a page menu'''
 
@@ -693,7 +798,7 @@ class PageIndexFunction(TemplateFunction):
 
 				if page == self._page:
 					# Current page is marked with the strong style
-					builder.start('strong')
+					builder.start('strong', {'_class': 'activepage'}) # HACK - used by Html output
 					builder.data(page.basename)
 					builder.end('strong')
 				else:
@@ -840,17 +945,33 @@ class PageProxy(object):
 	@property
 	def body(self):	return self._treeproxy().body
 
+	@property
+	def content(self): return self._treeproxy().content
+
 	@property
 	def parts(self): return None # TODO split in parts and return ParseTreeProxy objects
 
+	@property
+	def has_links(self):
+		return len(list(self._page.get_links())) > 0
+
 	@property
 	def links(self):
 		for type, name, _ in self._page.get_links():
 			if type == 'page':
-				page = self._notebook.get_page(name)
-				yield PageProxy(
-					self._notebook, page,
-					self._format, self._linker, self._options)
+				try:
+					path = self._notebook.resolve_path(name, self._page)
+					if path:
+						page = self._notebook.get_page(path)
+						yield PageProxy(
+							self._notebook, page,
+							self._format, self._linker, self._options)
+				except:
+					logger.exception('Error while exporting')
+
+	@property
+	def has_backlinks(self):
+		return len(list(self._notebook.index.list_links(self._page, LINK_DIR_BACKWARD))) > 0
 
 	@property
 	def backlinks(self):
@@ -859,6 +980,16 @@ class PageProxy(object):
 			source = self._notebook.get_page(link.source)
 			yield PageProxy(self._notebook, source, self._format, self._linker, self._options)
 
+	@property
+	def has_attachments(self):
+		return len(list(self._notebook.get_attachments_dir(self._page).list())) > 0
+
+	@property
+	def attachments(self):
+		dir = self._notebook.get_attachments_dir(self._page)
+		for basename in dir.list():
+			yield FilePathProxy(dir.file(basename), "./"+basename)
+
 
 class ParseTreeProxy(object):
 
@@ -891,6 +1022,13 @@ class ParseTreeProxy(object):
 
 			return ''.join(dumper.dump(body))
 
+	@property
+	def content(self):
+		if not self._tree:
+			return None
+		else:
+			return self._dump(self._tree)
+
 	def _split_head(self, tree):
 		if not hasattr(self, '_servered_head'):
 			elements = tree.getroot().getchildren()
@@ -904,3 +1042,44 @@ class ParseTreeProxy(object):
 				self._servered_head = (None, tree)
 
 		return self._servered_head
+
+	def _dump(self, tree):
+		format = self._pageproxy._format
+		linker = self._pageproxy._linker
+		if linker:
+			linker.set_path(self._pageproxy._page)
+
+		dumper = format.Dumper(
+			linker=linker,
+			template_options=self._pageproxy._options )
+
+		return ''.join(dumper.dump(tree))
+
+
+class FilePathProxy(object):
+	'''Proxy for L{File} and L{Dir} objects'''
+
+	# Keep in mind that "path" can refer to attachment in
+	# actual notebook, but after export we should refer to
+	# new copy of that item.
+	# So do not allow "file.uri", but use "url(file)" instead
+
+	def __init__(self, path, href=None):
+		self._path = path
+		self._href = href
+
+	@property
+	def path(self):
+		return self._href or self._path.user_path()
+
+	@property
+	def basename(self):
+		return self._path.basename
+
+	@property
+	def mtime(self):
+		return datetime.datetime.fromtimestamp(float(self._href.mtime()))
+
+	@property
+	def size(self):
+		return format_file_size(self._path.size())
